load [file join ../../Library/AT91Boot_TCL.dll] At91boot_tcl

################################################################################
#  proc ::CheckError
################################################################################
proc CheckError { err } {

if {[expr !($err == 0)]} {
	error [format "Error 0x%x" $err]
}

return
}



set DevList(0)    ""

puts "TCL_Scan"

set i 0
foreach {name} [TCL_Scan] {
    set DevList($i) $name
    puts "DevList($i): $DevList($i)"
    incr i 1
}
 
puts "TCL_Open"
set h_handle 0
set i 0
puts "DevList($i): $DevList($i)"
set h_handle [TCL_Open $DevList($i)]
puts [format "h_handle: %08X" $h_handle]

set error_code 0x0
 
##############################################################################
#"CAN CONFIGURATION
##############################################################################
variable CAN_ID_SPC_CMD_READ	   0x0
variable CAN_ID_SPC_CMD_WRITE	   0x1
variable CAN_SET_BAUDRATE	   0x3
variable CAN_CONNECT_TARGET	   0x4
variable CAN_DISCONNECT_TARGET	   0x5

set param 0x0
set baudrate 1000

puts "Set CAN Baudrate"
TCL_CAN_Configure $h_handle $CAN_SET_BAUDRATE $param $baudrate error_code
::CheckError $error_code 

puts "Connect Target"
TCL_CAN_Configure $h_handle $CAN_CONNECT_TARGET $param $baudrate error_code
::CheckError $error_code 

puts "Disconnect Target"
TCL_CAN_Configure $h_handle $CAN_DISCONNECT_TARGET $param $baudrate error_code
::CheckError $error_code 

puts "Re-connect Target"
TCL_CAN_Configure $h_handle $CAN_CONNECT_TARGET $param $baudrate error_code
::CheckError $error_code 

# #"" DO NOT FORGET TO PROGRAM EFC_FMR register !!!!
variable PROG_FMR    	0x00480100
variable AT91C_EFC_FMR  0xFFFFFF60
puts "Configure EFC_FMR"
TCL_Write_Int $h_handle $PROG_FMR $AT91C_EFC_FMR error_code
::CheckError $error_code 

##############################################################################
#"TEST READ FUNCTIONS
##############################################################################
set addrDBGUChipID 0xFFFFF240

set ChipID [TCL_Read_Int $h_handle $addrDBGUChipID error_code]
::CheckError $error_code 
puts [format "ChipID Int: %08X" $ChipID]

set ChipID [TCL_Read_Short $h_handle $addrDBGUChipID error_code]
::CheckError $error_code 
puts [format "ChipID Short: %04X" $ChipID]

set ChipID [TCL_Read_Byte $h_handle $addrDBGUChipID error_code]
::CheckError $error_code 
puts [format "ChipID Byte: %02X" $ChipID]

#############################################################################
#"TEST READ/WRITE FUNCTIONS
##############################################################################
set addrSRAM 0x200000

set Value    0xCAFEDECA
puts [format "Write int: %08X" $Value]
TCL_Write_Int $h_handle $Value $addrSRAM error_code
::CheckError $error_code 
set ChipID [TCL_Read_Int $h_handle $addrSRAM error_code]
::CheckError $error_code 
puts [format "Read Int: %08X" $ChipID]

set Value    0xBEAD
puts [format "Write short: %08X" $Value]
TCL_Write_Short $h_handle $Value $addrSRAM error_code
::CheckError $error_code 
set ChipID [TCL_Read_Short $h_handle $addrSRAM error_code]
::CheckError $error_code 
puts [format "Read Short: %08X" $ChipID]

set Value    0xA5
puts [format "Write byte: %08X" $Value]
TCL_Write_Byte $h_handle $Value $addrSRAM error_code
::CheckError $error_code 
set ChipID [TCL_Read_Byte $h_handle $addrSRAM error_code]
::CheckError $error_code 
puts [format "Read Byte: %08X" $ChipID]

################################################################################
# TRY FLASH MouseUSBSAM7X.bin file into SAM7X256 internal flash
################################################################################
variable fileName              "MouseUSBSAM7X.bin"
variable fileAddr              0x0

global valueOfDataForSendFile

puts "Send File $fileName"
if { [catch {set f [open $fileName r]}] } {
    set valueOfDataForSendFile 0
    puts stderr "-E- Can't open file $fileName"
    return
}
    
fconfigure $f -translation binary
    
set size [file size $fileName]
puts "-I- File size = $size byte(s)"
set valueOfDataForSendFile [read $f $size]
    
close $f
if { [catch {TCL_SAM7X256_Send_Flash $h_handle $fileAddr valueOfDataForSendFile $size error_code}] } {
    puts stderr "-E- Can't send data, error in connection"
    set valueOfDataForSendFile 0
}
    
set valueOfDataForSendFile 0
    

puts "Disconnect Target"
TCL_CAN_Configure $h_handle $CAN_DISCONNECT_TARGET $param $baudrate error_code
::CheckError $error_code

puts "TCL_Close"
TCL_Close $h_handle

puts "End Test"
