// OLE_without_MFC.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <stdio.h>
using namespace AT91BOOTDLL_Lib;

#include "FCError.h"

typedef int AT91BOOT_HANDLE_TYPE;

#define	DBGU_CIDR_MASK ~0x000F001F	// Remove version and SRAMSIZ (bug DBGU_CIDR S128 et S256 rev C)
#define PROG_FMR	0x00480100		// EFC FMR configuration for 48MHz 

int main(int argc, char* argv[])
{
	if (FAILED(CoInitialize(NULL)))
		return 0;

	try
	{
		int i, uValue, error_code;
		unsigned char *pTest[10];
			
		// COM Object Creation
		IAT91BootDLLPtr pAT91BootDLL(__uuidof(AT91BootDLL));

		AT91BOOT_HANDLE_TYPE h_handle;

		// MANDATORY: Allocate memory for each string in the table
		for (i=0; i<10; i++)
			pTest[i] = (unsigned char *)malloc(100);

		// Scan all devices connected
		pAT91BootDLL->AT91Boot_Scan((unsigned char *)pTest);

		// Print all devices connected
		i= 0;
		while (pTest[i] != 0)
		{
			printf("Device %d = %s\n\r", i, pTest[i]);
			i++;
		}
		
		// Open first Device of the list
		pAT91BootDLL->AT91Boot_Open(pTest[0], &h_handle);

		if (h_handle != 0)
		{
			// Read DBGU Chip ID
			pAT91BootDLL->AT91Boot_Read_Int(h_handle, &uValue, 0xFFFFF240, NULL);
		
			// Assume Target is a SAM7S256 (SAM7S-EK)
			// Otherwise, modify that part by yourself, it will be a good training! ;-)
			if ((uValue & DBGU_CIDR_MASK) == (0x270B0940 & DBGU_CIDR_MASK))
			{
				HANDLE hFile = {NULL};
				DWORD dwSize, bytes_read;
				unsigned char *buffer;
				
				// Open the file and get ready to read it.
				// Don't be confused by the CreateFile() function,
				// we're only opening an existing file.
				hFile = CreateFile("BasicMouse.bin", GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE,
					NULL, OPEN_EXISTING,
					FILE_FLAG_SEQUENTIAL_SCAN, NULL);
				if(hFile == INVALID_HANDLE_VALUE)
				{
					printf("Failed to open the file.\n\n");
					return -1;
				}
				// Get the file size so we know how long to make our buffer.
				dwSize = GetFileSize(hFile, NULL);
				// Create a CString buffer of the proper length
				// and fill it with spaces.
				buffer = (unsigned char *)malloc(dwSize);
				// Read the file into the buffer
				ReadFile(hFile, buffer, dwSize, &bytes_read, NULL);
				// We're done with the file handle so close it.
				CloseHandle(hFile);

				// MANDATORY: Program EFC_FMR register
				// Assume Target is at 48MHz
				pAT91BootDLL->AT91Boot_Write_Int(h_handle, PROG_FMR, 0xFFFFFF60, &error_code);
				if (error_code != AT91C_BOOT_DLL_OK)
					goto exit;
	
				// Send File into SAM7S256 Flash
				pAT91BootDLL->AT91Boot_SAM7S256_Send_Flash(h_handle, 0, buffer, dwSize, &error_code);
				if (error_code != AT91C_BOOT_DLL_OK)
					goto exit;

exit:
				// Release buffer
				free(buffer);
			}
			

			// Close Device
			pAT91BootDLL->AT91Boot_Close(h_handle);
		}

		// Free Allocated memory
		for (i=0; i<10; i++)
			free(pTest[i]);
	}

	catch (const _com_error& Err)
	{
		printf("Error: %d\n\r", Err.ErrorMessage());
	}

	CoUninitialize();
	return 0;
}