#  ----------------------------------------------------------------------------
#          ATMEL Microcontroller Software Support  -  ROUSSET  -
#  ----------------------------------------------------------------------------
#  Copyright (c) 2006, Atmel Corporation
#
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#  this list of conditions and the disclaiimer below.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#  this list of conditions and the disclaimer below in the documentation and/or
#  other materials provided with the distribution. 
#
#  Atmel's name may not be used to endorse or promote products derived from
#  this software without specific prior written permission. 
#
#  DISCLAIMER: THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
#  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
#  DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
#  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
#  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#  ----------------------------------------------------------------------------


if { [ catch { source "$libPath(extLib)/$target(board)/SRAM.tcl"} errMsg] } {
    messageDialg  error.gif "Memory Description File not found:\n$errMsg" "File not found" ok
    exit
}

if { [ catch { source "$libPath(extLib)/$target(board)/SDRAM.tcl"} errMsg] } {
    messageDialg  error.gif "Memory Description File not found:\n$errMsg" "File not found" ok
    exit
}

if { [ catch { source "$libPath(extLib)/$target(board)/NANDFLASH.tcl"} errMsg] } {
    messageDialg  error.gif "Memory Description File not found:\n$errMsg" "File not found" ok
    exit
}

if { [ catch { source "$libPath(extLib)/$target(board)/DATAFLASH.tcl"} errMsg] } {
    messageDialg  error.gif "Memory Description File not found:\n$errMsg" "File not found" ok
    exit
}


array set memoryAlgo {
    "SRAM"                    "::at91cap9_sram"
    "SDRAM"                   "::at91cap9_sdram"
    "DataFlash AT45DB/DCB"    "::at91cap9_dataflash"
    "NandFlash"               "::at91cap9_nandflash"
    "Peripheral"              "::at91cap9_peripheral"
    "ROM"                     "::at91cap9_rom"
    "REMAP"                   "::at91cap9_remap"
    "LCD"                     "::at91cap9_lcd"
}

array set at91cap9_sram {
    dftDisplay  1
    dftDefault  1
    dftAddress  0x100000
    dftSize     0x8000
    dftSend     "SRAM::sendFile"
    dftReceive  "SRAM::receiveFile"
    dftScripts  ""
}

array set at91cap9_sdram {
    dftDisplay  1
    dftDefault  0
    dftAddress  0x70000000
    dftSize     0x8000000
    dftSend     "SDRAM::sendFile"
    dftReceive  "SDRAM::receiveFile"
    dftScripts  "::at91cap9_sdram_scripts"
}

array set at91cap9_sdram_scripts {
    "Enable SDRAM 48 Hz (default)"   "SDRAM::initSDRAM_48"
    "Enable SDRAM 100 MHz"           "SDRAM::initSDRAM_100"
    "Enable SDRAM 133 MHz"           "SDRAM::initSDRAM_133"
}


array set at91cap9_nandflash {
    dftDisplay  1
    dftDefault  0
    dftAddress  0x0
    dftSize     "$NANDFLASH::MemorySize"
    dftSend     "NANDFLASH::sendFile"
    dftReceive  "NANDFLASH::receiveFile"
    dftScripts  "::at91cap9_nandflash_scripts"
}

array set at91cap9_nandflash_scripts {
    "NandFlash Init"                "NANDFLASH::Init"
    "NandFlash Send Boot File"      "NANDFLASH::sendBootFileGUI"
    "NandFlash Erase All"           "NANDFLASH::EraseAllNandFlash"
    "NandFlash List Bad Blocks"     "NANDFLASH::BadBlockList"
}

array set at91cap9_dataflash {
    dftDisplay  1
    dftDefault  0
    dftAddress  0x0
    dftSize     "$DATAFLASH::MemorySize"
    dftSend     "DATAFLASH::sendFile"
    dftReceive  "DATAFLASH::receiveFile"
    dftScripts  "::at91cap9_dataflash_scripts"
}

array set at91cap9_dataflash_scripts {
    "Enable Dataflash on CS0"   "DATAFLASH::SelectDataflash AT91C_SPI0_CS0"
    "Erase All"                 "DATAFLASH::EraseAllDataFlash"
}

array set at91cap9_peripheral {
    dftDisplay  0
    dftDefault  0
    dftAddress  0xFF000000
    dftSize     0x01000000
    dftSend     ""
    dftReceive  ""
    dftScripts  ""
}

array set at91cap9_rom {
    dftDisplay  0
    dftDefault  0
    dftAddress  0x400000
    dftSize     0x8000
    dftSend     ""
    dftReceive  ""
    dftScripts  ""
}

array set at91cap9_remap {
    dftDisplay  0
    dftDefault  0
    dftAddress  0x00000000
    dftSize     0x8000
    dftSend     ""
    dftReceive  ""
    dftScripts  ""
}

array set at91cap9_lcd {
    dftDisplay  0
    dftDefault  0
    dftAddress  0x500000
    dftSize     0x8000
    dftSend     ""
    dftReceive  ""
    dftScripts  ""
}

