#  ----------------------------------------------------------------------------
#          ATMEL Microcontroller Software Support  -  ROUSSET  -
#  ----------------------------------------------------------------------------
#  Copyright (c) 2006, Atmel Corporation
#
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#  this list of conditions and the disclaiimer below.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#  this list of conditions and the disclaimer below in the documentation and/or
#  other materials provided with the distribution. 
#
#  Atmel's name may not be used to endorse or promote products derived from
#  this software without specific prior written permission. 
#
#  DISCLAIMER: THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
#  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
#  DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
#  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
#  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#  ----------------------------------------------------------------------------


################################################################################
################################################################################
## NAMESPACE DATAFLASH
################################################################################
################################################################################
namespace eval DATAFLASH {
    variable StartAddress        [expr 0x0]
    # Minimal Size AT45DB011B
    variable MemorySize          [expr 132*1024]
    variable MaxBootSize         [expr 156*1024]
    
    variable AT91C_SPI0_PCS0_DATAFLASH     0xE
    variable AT91C_DATAFLASH_TIMEOUT       0x100
    
    variable bufferAddr         0x20020010
    variable cmdAddr            0x20020000
    variable MASTER_CLOCK       48000000
    variable DATAFLASH_CLK      8000000
    # Dataflash Setup Time
    variable DATAFLASH_TCSS     [expr 0xc << 16]
    # Dataflash Hold Time
    variable DATAFLASH_TCHS     [expr 0x0 << 24]
    
    # Read Commands
    variable DB_CONTINUOUS_ARRAY_READ   0xE8
    variable DB_PAGE_READ               0xD2
    variable DB_BUF1_READ               0xD4
    variable DB_BUF2_READ               0xD6
    variable DB_STATUS                  0xD7
    
    # Write + Erase Commands
    variable DB_BUF1_WRITE              0x84
    variable DB_BUF2_WRITE              0x87
    variable DB_BUF1_PAGE_ERASE_PGM     0x83
    variable DB_BUF2_PAGE_ERASE_PGM     0x86
    variable DB_BUF1_PAGE_PGM           0x88
    variable DB_BUF2_PAGE_PGM           0x89
    variable DB_PAGE_ERASE              0x81
    variable DB_BLOCK_ERASE             0x50
    variable DB_PAGE_PGM_BUF1           0x82
    variable DB_PAGE_PGM_BUF2           0x85
    
    # Additional Commands
    variable DB_PAGE_2_BUF1_TRF         0x53
    variable DB_PAGE_2_BUF2_TRF         0x55
    
    variable GET_STATUS                 0x0F
    variable BUSY                       0x1
    variable IDLE                       0x0
    
    variable array ArrayDeviceID
    array set ArrayDeviceID {
        0x0C    AT45DB011
        0x14    AT45DB021
        0x1C    AT45DB041
        0x24    AT45DB081
        0x2C    AT45DB161
        0x34    AT45DB321
        0x3C    AT45DB642
        0x10    AT45DB1282
        0x18    AT45DB2562
        0x20    AT45DB5122
    }
    
    variable array deviceAT45
    array set deviceAT45 {
        pages_number 0
        pages_size   0
        page_offset  0
        byte_mask    0
    }
    
    variable array DataflashDesc
    array set DataflashDesc {
        tx_cmd_pt 0
        tx_cmd_size 0
        rx_cmd_pt 0
        rx_cmd_size 0
        tx_data_pt 0
        tx_data_size 0
        rx_data_pt 0
        rx_data_size 0
        DataFlash_state 0
        command0 0
        command1 0
        command2 0
        command3 0
        command4 0
        command5 0
        command6 0
        command7 0
        command8 0
    }
    
    # Parameters used by the C binary project: pageBuf is the page content (size of a page)
    variable comType                   0x200013EC
    variable df_dest                   0x200013F0    
    variable pageNb                    0x200013F4
    variable pageSize                  0x200013F8
    variable pageOffset                0x200013FC    
    variable pageBuf                   0x20001400
   
    # Link address & Binary name in the C project
    variable fileAddr                  0x20000000
    variable fileDFWrite               "SAM-BA-DataFlashWrite.bin"
    variable fileDFRead                "SAM-BA-DataFlashRead.bin"

    variable DATAFLASH_initialized     [expr 0]
}

################################################################################
#  proc DATAFLASH::SpiWrite
################################################################################
proc DATAFLASH::SpiWrite {} {
    
    global AT91C_SPI_PTCR AT91C_PDC_RXTDIS AT91C_PDC_TXTDIS AT91C_PDC_RXTEN AT91C_PDC_TXTEN
    global AT91C_SPI_SR AT91C_SPI_RXBUFF
    global AT91C_SPI_RPR AT91C_SPI_TPR AT91C_SPI_RCR AT91C_SPI_TCR
    global AT91C_SPI_TNPR AT91C_SPI_TNCR AT91C_SPI_RNPR AT91C_SPI_RNCR
    variable BUSY
    variable IDLE
    variable DataflashDesc
    
    global target
    set dummy_err 0

    TCL_Write_Int $target(handle) [expr $AT91C_PDC_TXTDIS | $AT91C_PDC_RXTDIS ] $AT91C_SPI_PTCR dummy_err
    
    # Initialize the Transmit and Receive Pointer"
    TCL_Write_Int $target(handle) $DataflashDesc(rx_cmd_pt) $AT91C_SPI_RPR dummy_err
    TCL_Write_Int $target(handle) $DataflashDesc(tx_cmd_pt) $AT91C_SPI_TPR dummy_err
    
    # Intialize the Transmit and Receive Counters"
    TCL_Write_Int $target(handle) $DataflashDesc(rx_cmd_size) $AT91C_SPI_RCR dummy_err
    TCL_Write_Int $target(handle) $DataflashDesc(tx_cmd_size) $AT91C_SPI_TCR dummy_err 
    
    if {$DataflashDesc(tx_data_size) != 0} {
        # Initialize the Next Transmit and Next Receive Pointer"
        TCL_Write_Int $target(handle) $DataflashDesc(rx_data_pt) $AT91C_SPI_RNPR dummy_err
        TCL_Write_Int $target(handle) $DataflashDesc(tx_data_pt) $AT91C_SPI_TNPR dummy_err
        
        # Intialize the Next Transmit and Next Receive Counters"
        TCL_Write_Int $target(handle) $DataflashDesc(rx_data_size) $AT91C_SPI_RNCR dummy_err
        TCL_Write_Int $target(handle) $DataflashDesc(tx_data_size) $AT91C_SPI_TNCR dumm_err
    }
    
    TCL_Write_Int $target(handle) [expr $AT91C_PDC_TXTEN + $AT91C_PDC_RXTEN] $AT91C_SPI_PTCR dummy_err
    
    while {[expr [TCL_Read_Int $target(handle) $AT91C_SPI_SR dummy_err] & $AT91C_SPI_RXBUFF] != $AT91C_SPI_RXBUFF} {}
}

################################################################################
#  proc DATAFLASH::GetStatus
################################################################################
proc DATAFLASH::GetStatus {} {
    
    variable DB_STATUS
    variable GET_STATUS
    variable bufferAddr
    variable cmdAddr
    variable DataflashDesc
    
    global target
    set dummy_err 0

    set DataflashDesc(command0) $DB_STATUS
    TCL_Write_Int $target(handle) $DataflashDesc(command0) $cmdAddr dummy_err
    set DataflashDesc(command1) 0
    
    set DataflashDesc(DataFlash_state) $GET_STATUS
    set DataflashDesc(tx_data_size)  0
    set DataflashDesc(tx_cmd_pt)     $cmdAddr
    set DataflashDesc(rx_cmd_pt)     $cmdAddr
    set DataflashDesc(rx_cmd_size)   2
    set DataflashDesc(tx_cmd_size)   2
    
    DATAFLASH::SpiWrite
    set DataflashDesc(DataFlash_state) [TCL_Read_Byte $target(handle) [expr $DataflashDesc(rx_cmd_pt) + 1] dummy_err]
}

################################################################################
#  proc DATAFLASH::WaitReady
################################################################################
proc DATAFLASH::WaitReady {timeout} {
    
    variable DataflashDesc
    variable IDLE
    
    set Temptime 0
    
    while {$Temptime < $timeout} {
        DATAFLASH::GetStatus
        incr Temptime
        if {[expr $DataflashDesc(DataFlash_state) & 0x80] == 0x80} {
            return 1
        }
    }
    
    if {$Temptime >= $timeout} {
        puts "TimeOut error"
        return 0
    }
}

################################################################################
#  proc DATAFLASH::SendCommand
################################################################################
proc DATAFLASH::SendCommand { OpCode CmdSize DataflashAddress } {
    
    variable deviceAT45
    variable DataflashDesc
    variable cmdAddr
    
    global target
    set dummy_err 0

    # Compute the address to obtain page address and byte address
    # adr = ((DataflashAddress / (pDataFlash->pDevice->pages_size)) << pDataFlash->pDevice->page_offset) + (DataflashAddress % (pDataFlash->pDevice->pages_size));
    set adr [expr [expr [expr $DataflashAddress / $deviceAT45(pages_size)] << $deviceAT45(page_offset)] + [expr $DataflashAddress % $deviceAT45(pages_size)]]
    
    # Fill the  command  buffer */
    TCL_Write_Byte $target(handle) $OpCode $cmdAddr dummy_err
    if {$deviceAT45(pages_number) >= 16384} {
        TCL_Write_Byte $target(handle) [expr [expr $adr & 0x0F000000] >> 24 ] [expr $cmdAddr + 1] dummy_err
        TCL_Write_Byte $target(handle) [expr [expr $adr & 0x00FF0000] >> 16 ] [expr $cmdAddr + 2] dummy_err
        TCL_Write_Byte $target(handle) [expr [expr $adr & 0x0000FF00] >>  8 ] [expr $cmdAddr + 3] dummy_err
        TCL_Write_Byte $target(handle) [expr [expr $adr & 0x000000FF] >>  0 ] [expr $cmdAddr + 4] dummy_err
    } else {
        TCL_Write_Byte $target(handle) [expr [expr $adr & 0x00FF0000] >> 16 ] [expr $cmdAddr + 1] dummy_err
        TCL_Write_Byte $target(handle) [expr [expr $adr & 0x0000FF00] >>  8 ] [expr $cmdAddr + 2] dummy_err
        TCL_Write_Byte $target(handle) [expr [expr $adr & 0x000000FF] >>  0 ] [expr $cmdAddr + 3] dummy_err
        TCL_Write_Byte $target(handle) 0 [expr $cmdAddr + 4] dummy_err
    }
    TCL_Write_Byte $target(handle) [expr $cmdAddr + 5] 0 dummy_err
    TCL_Write_Byte $target(handle) [expr $cmdAddr + 6] 0 dummy_err
    TCL_Write_Byte $target(handle) [expr $cmdAddr + 7] 0 dummy_err
    
    # Initialize the SpiData structure for the spi write fuction */
    set DataflashDesc(tx_cmd_pt)    $cmdAddr
    set DataflashDesc(rx_cmd_pt)    $cmdAddr
    set DataflashDesc(rx_cmd_size)  $CmdSize
    set DataflashDesc(tx_cmd_size)  $CmdSize
    
    # Send the command
    DATAFLASH::SpiWrite
}

################################################################################
#  proc DATAFLASH::WriteBuffer
################################################################################
proc DATAFLASH::WriteBuffer { src buffer_dest SizeToWrite } {
    variable DataflashDesc
    variable deviceAT45
    variable bufferAddr
    variable cmdAddr
    variable DB_BUF1_WRITE
    
    global target
    set dummy_err 0

    # Fill the  command  buffer */
    TCL_Write_Byte $target(handle) $DB_BUF1_WRITE $cmdAddr dummy_err
    TCL_Write_Byte $target(handle) 0 [expr $cmdAddr + 1] dummy_err
    
    if {$deviceAT45(pages_number) >= 16384} {
        TCL_Write_Byte $target(handle) 0 [expr $cmdAddr + 2] dummy_err
        TCL_Write_Byte $target(handle) [expr [expr $buffer_dest & [expr $deviceAT45(byte_mask)]] >> 8] [expr $cmdAddr + 3] dummy_err
        TCL_Write_Byte $target(handle) [expr [expr $buffer_dest & 0x000000FF] >> 0] [expr $cmdAddr + 4] dummy_err
        set cmdsize 5
    } else {
        TCL_Write_Byte $target(handle) [expr [expr $buffer_dest & [expr $deviceAT45(byte_mask)]] >> 8] [expr $cmdAddr + 2] dummy_err
        TCL_Write_Byte $target(handle) [expr [expr $buffer_dest & 0x000000FF] >> 0] [expr $cmdAddr + 3] dummy_err
        TCL_Write_Byte $target(handle) 0 [expr $cmdAddr + 4] dummy_err
        set cmdsize 4
    }
    
    # Copy in SRAM the content of the page to be written
    TCL_Write_Data $target(handle) $bufferAddr src $SizeToWrite dummy_err
    # Initialize the SpiData structure for the spi write fuction */
    set DataflashDesc(rx_cmd_pt)     $cmdAddr
    set DataflashDesc(rx_cmd_size)   $cmdsize
    set DataflashDesc(tx_cmd_pt)     $cmdAddr
    set DataflashDesc(tx_cmd_size)   $cmdsize
    set DataflashDesc(tx_data_pt)    $bufferAddr
    set DataflashDesc(tx_data_size)  $SizeToWrite
    set DataflashDesc(rx_data_pt)    $bufferAddr
    set DataflashDesc(rx_data_size)  $SizeToWrite
    
    # Send the command
    DATAFLASH::SpiWrite
}

################################################################################
#  proc DATAFLASH::WriteBuffer2Main
################################################################################
proc DATAFLASH::WriteBuffer2Main { dest } {
    variable DataflashDesc
    variable deviceAT45
    variable DB_BUF1_PAGE_PGM
    
    # No data to receive or transmit
    set DataflashDesc(rx_data_size)       0
    set DataflashDesc(tx_data_size)       0
    
    set cmdsize 4
    if {$deviceAT45(pages_number) >= 16384} {
        set cmdsize 5
    }
    
    # Send the command to the dataflash
    set returnValue [DATAFLASH::SendCommand $DB_BUF1_PAGE_PGM $cmdsize $dest]
    return $returnValue
}

################################################################################
#  proc DATAFLASH::MainMemory2Buffer
################################################################################
proc DATAFLASH::MainMemory2Buffer { page } {
    variable deviceAT45
    variable DB_PAGE_2_BUF1_TRF
    
    # No data to receive or transmit
    set DataflashDesc(rx_data_size)       0
    set DataflashDesc(tx_data_size)       0
    
    set cmdsize 4
    if {$deviceAT45(pages_number) >= 16384} {
        set cmdsize 5
    }
    # Send the command to the dataflash
    set returnValue [DATAFLASH::SendCommand $DB_PAGE_2_BUF1_TRF $cmdsize [expr $page * $deviceAT45(pages_size)]]
    return $returnValue
}

################################################################################
#  proc DATAFLASH::PageErase
################################################################################
proc DATAFLASH::PageErase { page } {
    variable deviceAT45
    variable DB_PAGE_ERASE
    
    set cmdsize 4
    if {$deviceAT45(pages_number) >= 16384} {
        set cmdsize 5
    }
    
    # Send the command to the dataflash
    set returnValue [DATAFLASH::SendCommand $DB_PAGE_ERASE $cmdsize [expr $page * $deviceAT45(pages_size)]]
    return $returnValue
}

################################################################################
#  proc DATAFLASH::BlockErase
################################################################################
proc DATAFLASH::BlockErase { block } {
    variable deviceAT45
    variable DB_BLOCK_ERASE
    
    set blocknb [expr $deviceAT45(pages_number) / 8 - 1]
    if { [expr $block < 0] || [expr $block > $blocknb] } {
        error AT91C_ERR3
    }
    
    set cmdsize 4
    if {$deviceAT45(pages_number) >= 16384} {
        set cmdsize 5
    }
    
    # Send the command to the dataflash
    DATAFLASH::SendCommand $DB_BLOCK_ERASE $cmdsize [expr $block * 8 * $deviceAT45(pages_size)]
}

################################################################################
#  proc DATAFLASH::PagePgmBuf (not used)
################################################################################
proc DATAFLASH::PagePgmBuf { src dest SizeToWrite } {
    
    variable DataflashDesc
    variable DB_PAGE_PGM_BUF1
    variable bufferAddr
    variable cmdAddr
    variable deviceAT45
    
    global target
    set dummy_err 0

    set DataflashDesc(rx_data_size)   $SizeToWrite
    set DataflashDesc(tx_data_size)   $SizeToWrite
    # Copy in SRAM the content of the page to be written
    TCL_Write_Data $target(handle) $bufferAddr src $SizeToWrite dummy_err
    set DataflashDesc(tx_cmd_pt)      $cmdAddr
    set DataflashDesc(rx_cmd_pt)      $cmdAddr
    set DataflashDesc(rx_data_pt)     $bufferAddr
    set DataflashDesc(tx_data_pt)     $bufferAddr
    
    set cmdsize 4
    if {$deviceAT45(pages_number) >= 16384} {
        set cmdsize 5
    }
    
    # Send the command to the dataflash
    set returnValue [DATAFLASH::SendCommand $DB_PAGE_PGM_BUF1 $cmdsize $dest]
    return $returnValue
}

################################################################################
#  proc DATAFLASH::ReadBuffer
################################################################################
proc DATAFLASH::ReadBuffer { src SizeToRead } {
    variable DataflashDesc
    variable deviceAT45
    variable bufferAddr
    variable cmdAddr
    variable DB_BUF1_READ
    
    global target
    set dummy_err 0

    # Fill the  command  buffer */
    TCL_Write_Byte $target(handle) $DB_BUF1_READ $cmdAddr dummy_err
    TCL_Write_Byte $target(handle) 0 [expr $cmdAddr + 1] dummy_err
    
    if {$deviceAT45(pages_number) >= 16384} {
        TCL_Write_Byte $target(handle) 0 [expr $cmdAddr + 2] dummy_err
        TCL_Write_Byte $target(handle) [expr [expr $src & [expr $deviceAT45(byte_mask)]] >> 8] [expr $cmdAddr + 3] dummy_err
        TCL_Write_Byte $target(handle) [expr [expr $src & 0x000000FF] >>  0 ] [expr $cmdAddr + 4] dummy_err
        set cmdsize 6
    } else {
        TCL_Write_Byte $target(handle) [expr [expr $src & [expr $deviceAT45(byte_mask)]] >> 8] [expr $cmdAddr + 2] dummy_err
        TCL_Write_Byte $target(handle) [expr [expr $src & 0x000000FF] >>  0 ] [expr $cmdAddr + 3] dummy_err
        TCL_Write_Byte $target(handle) 0 [expr $cmdAddr + 4] dummy_err
        set cmdsize 5
    }
    
    # Initialize the SpiData structure for the spi write fuction */
    set DataflashDesc(rx_cmd_pt)     $cmdAddr
    set DataflashDesc(rx_cmd_size)   $cmdsize
    set DataflashDesc(tx_cmd_pt)     $cmdAddr
    set DataflashDesc(tx_cmd_size)   $cmdsize
    set DataflashDesc(tx_data_pt)    $bufferAddr
    set DataflashDesc(tx_data_size)  $SizeToRead
    set DataflashDesc(rx_data_pt)    $bufferAddr
    set DataflashDesc(rx_data_size)  $SizeToRead
    
    # Send the command
    DATAFLASH::SpiWrite
}

################################################################################
#  proc DATAFLASH::PageRead
################################################################################
proc DATAFLASH::PageRead { src sizeToRead } {
    
    variable DataflashDesc
    variable bufferAddr
    variable DB_PAGE_READ
    variable cmdAddr
    
    set DataflashDesc(rx_data_size)  $sizeToRead
    set DataflashDesc(tx_data_size)  $sizeToRead
    set DataflashDesc(tx_data_pt)    $bufferAddr
    set DataflashDesc(rx_data_pt)    $bufferAddr
    
    # Send the command to the dataflash
    set returnValue [DATAFLASH::SendCommand $DB_PAGE_READ 8 $src]
    return $returnValue
}

################################################################################
#  proc DATAFLASH::PartialPageWrite
################################################################################
proc DATAFLASH::PartialPageWrite { src dest sizeToWrite } {
    variable deviceAT45
    variable AT91C_DATAFLASH_TIMEOUT
    
    set page [expr $dest / $deviceAT45(pages_size)]
    set AdrInPage [expr $dest % $deviceAT45(pages_size)]
    DATAFLASH::MainMemory2Buffer $page
    set returnValue [DATAFLASH::WaitReady $AT91C_DATAFLASH_TIMEOUT]
    DATAFLASH::WriteBuffer $src $AdrInPage $sizeToWrite
    set returnValue [DATAFLASH::WaitReady $AT91C_DATAFLASH_TIMEOUT]
    DATAFLASH::PageErase $page
    set returnValue [DATAFLASH::WaitReady $AT91C_DATAFLASH_TIMEOUT]
    DATAFLASH::WriteBuffer2Main [expr $page * $deviceAT45(pages_size)]
    set returnValue [DATAFLASH::WaitReady $AT91C_DATAFLASH_TIMEOUT]
}

################################################################################
#  proc DATAFLASH::ContinuousRead (not used)
################################################################################
proc DATAFLASH::ContinuousRead { src dest sizeToRead } {
    variable DataflashDesc
    variable bufferAddr
    variable DB_CONTINUOUS_ARRAY_READ
    variable cmdAddr
    
    set DataflashDesc(rx_data_size)  $sizeToRead
    set DataflashDesc(tx_data_size)  $sizeToRead
    set DataflashDesc(tx_data_pt)    $dest
    set DataflashDesc(rx_data_pt)    $dest
    
    # Send the command to the dataflash
    set returnValue [DATAFLASH::SendCommand $DB_CONTINUOUS_ARRAY_READ 8 $src]
    return $returnValue
}

################################################################################
#  proc DATAFLASH::Write
################################################################################
proc DATAFLASH::Write { dest sizeToWrite File } {
    global valueOfDataForSendFile
    variable deviceAT45
    variable AT91C_DATAFLASH_TIMEOUT
    
    if { [expr $dest < 0] || [expr $sizeToWrite < 0] } {
        error AT91C_ERR1
    }
    
    if { [expr $dest + $sizeToWrite] > [expr $deviceAT45(pages_size) * $deviceAT45(pages_number)] } {
        error AT91C_ERR2
    }
    
    set AdrInPage [expr $dest % $deviceAT45(pages_size)]
    
    if {$AdrInPage} {
        set length [expr $deviceAT45(pages_size) - $AdrInPage]
        if {$sizeToWrite < $length} {
            set length $sizeToWrite
        }
        set valueOfDataForSendFile [read $File $length]
        DATAFLASH::PartialPageWrite $valueOfDataForSendFile $dest $length
        set returnValue [DATAFLASH::WaitReady $AT91C_DATAFLASH_TIMEOUT]
        
        set sizeToWrite [expr $sizeToWrite - $length]
        set dest [expr $dest + $length]
        #set valueOfDataForSendFile [expr $valueOfDataForSendFile + $length]
    }
    
    while { [expr $sizeToWrite - $deviceAT45(pages_size)] >= 0 } {
        set page [expr $dest / $deviceAT45(pages_size)]
        set valueOfDataForSendFile  [read $File $deviceAT45(pages_size)]
        
        DATAFLASH::WriteBuffer $valueOfDataForSendFile 0 $deviceAT45(pages_size)
        set returnValue [DATAFLASH::WaitReady $AT91C_DATAFLASH_TIMEOUT]
        DATAFLASH::PageErase $page
        set returnValue [DATAFLASH::WaitReady $AT91C_DATAFLASH_TIMEOUT]
        DATAFLASH::WriteBuffer2Main $dest
        set returnValue [DATAFLASH::WaitReady $AT91C_DATAFLASH_TIMEOUT]
        
        set sizeToWrite [expr $sizeToWrite - $deviceAT45(pages_size)]
        set dest [expr $dest + $deviceAT45(pages_size)]
        #set valueOfDataForSendFile  [expr $valueOfDataForSendFile +  $deviceAT45(pages_size)]
    }
    
    if {$sizeToWrite > 0} {
        set valueOfDataForSendFile  [read $File $sizeToWrite]
        DATAFLASH::PartialPageWrite $valueOfDataForSendFile $dest $sizeToWrite
        set returnValue [DATAFLASH::WaitReady $AT91C_DATAFLASH_TIMEOUT]
    }
    return 1
}

################################################################################
#  proc DATAFLASH::FastWrite
################################################################################
proc DATAFLASH::FastWrite { dest sizeToWrite File } {
    global valueOfDataForSendFile
    variable deviceAT45
    variable AT91C_DATAFLASH_TIMEOUT
    variable fileDFWrite
    variable fileAddr
    variable df_dest
    variable pageNb
    variable pageSize
    variable pageOffset
    variable pageBuf
    variable comType
    
    global libPath
    global target
    set dummy_err 0

    if { [expr $dest < 0] || [expr $sizeToWrite < 0] } {
        error AT91C_ERR1
    }
    
    if { [expr $dest + $sizeToWrite] > [expr $deviceAT45(pages_size) * $deviceAT45(pages_number)] } {
        error AT91C_ERR2
    }
    
    set AdrInPage [expr $dest % $deviceAT45(pages_size)]
    
    if {$AdrInPage} {
        set length [expr $deviceAT45(pages_size) - $AdrInPage]
        if {$sizeToWrite < $length} {
            set length $sizeToWrite
        }
        set valueOfDataForSendFile [read $File $length]
        DATAFLASH::PartialPageWrite $valueOfDataForSendFile $dest $length
        set returnValue [DATAFLASH::WaitReady $AT91C_DATAFLASH_TIMEOUT]
        
        set sizeToWrite [expr $sizeToWrite - $length]
        set dest [expr $dest + $length]
    }
    
    # Open DataFlash Write file
    set fileName "$libPath(extLib)/$target(board)/$fileDFWrite"
    if { [catch {set f [open $fileName r]}] } {
        puts stderr "-E- Can't open file $fileName"
        return -1
    }

    # Copy $fileName content into SRAM reserved memory (at address $fileAddr)
    fconfigure $f -translation binary
    set size [file size $fileName]
    set valueOfDataForSendFile [read $f $size]
    catch {TCL_Write_Data $target(handle) $fileAddr valueOfDataForSendFile $size dummy_err}
    close $f

    # Copy DataFlash Page Size and Page Offset into SRAM reserved memory
    TCL_Write_Int $target(handle) $target(comType) $comType dummy_err    
    TCL_Write_Int $target(handle) $deviceAT45(pages_number) $pageNb dummy_err
    TCL_Write_Int $target(handle) $deviceAT45(pages_size) $pageSize dummy_err
    TCL_Write_Int $target(handle) $deviceAT45(page_offset) $pageOffset dummy_err
    
    while {[expr $sizeToWrite - $deviceAT45(pages_size)] >= 0} {
        
        # Copy corresponding dest into SRAM reserved memory (at address $df_dest)
        TCL_Write_Int $target(handle) $dest $df_dest dummy_err
        set valueOfDataForSendFile [read $File $deviceAT45(pages_size)]
        # Copy corresponding page content into SRAM reserved memory (at address $pageBuf)
        catch {TCL_Write_Data $target(handle) $pageBuf valueOfDataForSendFile $deviceAT45(pages_size) dummy_err}
        # Branch on C binary
        catch {TCL_Go $target(handle) $fileAddr dummy_err}
        
        set sizeToWrite [expr $sizeToWrite - $deviceAT45(pages_size)]
        set dest [expr $dest + $deviceAT45(pages_size)]
    }

    if {$sizeToWrite > 0} {
        set valueOfDataForSendFile [read $File $sizeToWrite]
        DATAFLASH::PartialPageWrite $valueOfDataForSendFile $dest $sizeToWrite
        set returnValue [DATAFLASH::WaitReady $AT91C_DATAFLASH_TIMEOUT]
    }
    return 1
}

################################################################################
#  proc DATAFLASH::Read
################################################################################
proc DATAFLASH::Read { src sizeToRead File } {
    variable deviceAT45
    variable bufferAddr
    variable AT91C_DATAFLASH_TIMEOUT
    
    global target
    set dummy_err 0

    if { [expr $src < 0] || [expr $sizeToRead < 0] } {
        error AT91C_ERR1
    }
    
    if { [expr $src + $sizeToRead] > [expr $deviceAT45(pages_size) * $deviceAT45(pages_number)] } {
        error AT91C_ERR2
    }
    
    set AdrInPage [expr $src % $deviceAT45(pages_size)]
    
    if {$AdrInPage} {
        set length [expr $deviceAT45(pages_size) - $AdrInPage]
        if {$sizeToRead < $length} {
            set length $sizeToRead
        }
        
        DATAFLASH::PageRead $src $length
        set returnValue [DATAFLASH::WaitReady $AT91C_DATAFLASH_TIMEOUT]
        
        set sizeToRead [expr $sizeToRead - $length]
        set src [expr $src + $length]
        
        set result [TCL_Read_Data $target(handle) $bufferAddr $length dummy_err]
        puts -nonewline $File $result
    }
    
    while { [expr $sizeToRead - $deviceAT45(pages_size)] >= 0 } {
        
        DATAFLASH::PageRead $src $deviceAT45(pages_size)
        set returnValue [DATAFLASH::WaitReady $AT91C_DATAFLASH_TIMEOUT]
        
        set sizeToRead [expr $sizeToRead - $deviceAT45(pages_size)]
        set src [expr $src + $deviceAT45(pages_size)]
        
        set result [TCL_Read_Data $target(handle) $bufferAddr $deviceAT45(pages_size) dummy_err]
        puts -nonewline $File $result
    }
    
    if {$sizeToRead > 0} {
        DATAFLASH::PageRead $src $sizeToRead
        set returnValue [DATAFLASH::WaitReady $AT91C_DATAFLASH_TIMEOUT]
        
        set result [TCL_Read_Data $target(handle) $bufferAddr $sizeToRead dummy_err]
        puts -nonewline $File $result
    }
    return 1
}

################################################################################
#  proc DATAFLASH::FastRead
################################################################################
proc DATAFLASH::FastRead { src sizeToRead File } {
    variable deviceAT45
    variable bufferAddr
    variable AT91C_DATAFLASH_TIMEOUT
    variable fileDFRead
    variable fileAddr
    variable df_dest
    variable pageNb
    variable pageSize
    variable pageOffset
    variable pageBuf
    variable comType
    
    global libPath
    global target
    set dummy_err 0

    if { [expr $src < 0] || [expr $sizeToRead < 0] } {
        error AT91C_ERR1
    }
    
    if { [expr $src + $sizeToRead] > [expr $deviceAT45(pages_size) * $deviceAT45(pages_number)] } {
        error AT91C_ERR2
    }
       
    set AdrInPage [expr $src % $deviceAT45(pages_size)]
    
    if {$AdrInPage} {
        set length [expr $deviceAT45(pages_size) - $AdrInPage]
        if {$sizeToRead < $length} {
            set length $sizeToRead
        }
        
        DATAFLASH::PageRead $src $length
        set returnValue [DATAFLASH::WaitReady $AT91C_DATAFLASH_TIMEOUT]
        
        set sizeToRead [expr $sizeToRead - $length]
        set src [expr $src + $length]
        
        set result [TCL_Read_Data $target(handle) $bufferAddr $length dummy_err]
        puts -nonewline $File $result
    }
    
    # Open Data Flash Read file
    set fileName "$libPath(extLib)/$target(board)/$fileDFRead"
    if { [catch {set f [open $fileName r]}] } {
        puts stderr "-E- Can't open file $fileName"
        return -1
    }

    # Copy $fileName content into SRAM reserved memory (at address $fileAddr)
    fconfigure $f -translation binary
    set size [file size $fileName]
    set valueOfDataForSendFile [read $f $size]
    catch {TCL_Write_Data $target(handle) $fileAddr valueOfDataForSendFile $size dummy_err}
    close $f

    # Copy DataFlash Page Size and Page Offset into SRAM reserved memory
    TCL_Write_Int $target(handle) $target(comType) $comType dummy_err    
    TCL_Write_Int $target(handle) $deviceAT45(pages_number) $pageNb dummy_err
    TCL_Write_Int $target(handle) $deviceAT45(pages_size) $pageSize dummy_err
    TCL_Write_Int $target(handle) $deviceAT45(page_offset) $pageOffset dummy_err
    
    while {[expr $sizeToRead - $deviceAT45(pages_size)] >= 0} {
        
        # Copy corresponding dest into SRAM reserved memory (at address $df_dest)
        TCL_Write_Int $target(handle) $src $df_dest dummy_err
        # Branch on C binary
        catch {TCL_Go $target(handle) $fileAddr dummy_err}
        
        set sizeToRead [expr $sizeToRead - $deviceAT45(pages_size)]
        set src [expr $src + $deviceAT45(pages_size)]
        
        set result [TCL_Read_Data $target(handle) $pageBuf $deviceAT45(pages_size) dummy_err]
        puts -nonewline $File $result
    }

    if {$sizeToRead > 0} {
        DATAFLASH::PageRead $src $sizeToRead
        set returnValue [DATAFLASH::WaitReady $AT91C_DATAFLASH_TIMEOUT]
        
        set result [TCL_Read_Data $target(handle) $bufferAddr $sizeToRead dummy_err]
        puts -nonewline $File $result
    }
    return 1
}


################################################################################
#  proc DATAFLASH::Init
################################################################################
proc DATAFLASH::Init { } {

    # PIO
    global AT91C_PA11_NPCS0 AT91C_PA12_MISO AT91C_PA13_MOSI AT91C_PA14_SPCK
    global AT91C_PIOA_ASR AT91C_PIOA_BSR AT91C_PIOA_PDR
    
    # PMC
    global AT91C_PMC_PCER AT91C_ID_SPI
    
    # SPI0
    global AT91C_SPI_MR AT91C_SPI_MSTR AT91C_SPI_MODFDIS AT91C_SPI_PCS
    global AT91C_SPI_CR AT91C_SPI_SWRST
    global AT91C_SPI_CSR AT91C_SPI_NCPHA AT91C_SPI_DLYBS AT91C_SPI_DLYBCT
    global AT91C_SPI_PTCR AT91C_PDC_RXTEN AT91C_PDC_TXTEN AT91C_PDC_RXTDIS AT91C_PDC_TXTDIS
    global AT91C_SPI_TNPR AT91C_SPI_TNCR AT91C_SPI_RNPR AT91C_SPI_RNCR
    global AT91C_SPI_TPR AT91C_SPI_TCR AT91C_SPI_RPR AT91C_SPI_RCR
    
    variable MASTER_CLOCK
    variable DATAFLASH_CLK
    variable DATAFLASH_TCSS
    variable DATAFLASH_TCHS
    
    global target
    set dummy_err 0

    puts "-I- Init SPI for DataFlash interfaces"
    puts "-I- Configure SPI PIOs"
    set periphA [ expr $AT91C_PA11_NPCS0 | $AT91C_PA12_MISO | $AT91C_PA13_MOSI | $AT91C_PA14_SPCK ]
    set periphB 0
    TCL_Write_Int $target(handle) $periphA $AT91C_PIOA_ASR dummy_err
    TCL_Write_Int $target(handle) $periphB $AT91C_PIOA_BSR dummy_err
    TCL_Write_Int $target(handle) [expr $periphA | $periphB] $AT91C_PIOA_PDR dummy_err
    
    puts "-I- Enable PMC for SPI0"
    TCL_Write_Int $target(handle) [expr 1 << $AT91C_ID_SPI] $AT91C_PMC_PCER dummy_err
    
    puts "-I- Reset the SPI0"
    TCL_Write_Int $target(handle) $AT91C_SPI_SWRST $AT91C_SPI_CR dummy_err
    
    puts "-I- Configure SPI in Master Mode (no CS selected)"
    TCL_Write_Int $target(handle) [expr $AT91C_SPI_MSTR | $AT91C_SPI_MODFDIS | $AT91C_SPI_PCS] $AT91C_SPI_MR dummy_err
    
    puts "-I- Configure SPI CS0 for DataFlash AT45"
    TCL_Write_Int $target(handle) [expr $AT91C_SPI_NCPHA | [expr $AT91C_SPI_DLYBS & $DATAFLASH_TCSS] | [expr $AT91C_SPI_DLYBCT & $DATAFLASH_TCHS] |\
            [expr [expr $MASTER_CLOCK / $DATAFLASH_CLK] << 8]] [expr $AT91C_SPI_CSR + 4*0] dummy_err
    
    puts "-I- Disable the RX and TX PDC transfer requests"
    TCL_Write_Int $target(handle) [expr $AT91C_PDC_RXTDIS | $AT91C_PDC_TXTDIS] $AT91C_SPI_PTCR dummy_err
    
    puts "-I- Reset all Counter register Next buffer first"
    TCL_Write_Int $target(handle) 0 $AT91C_SPI_TNPR dummy_err
    TCL_Write_Int $target(handle) 0 $AT91C_SPI_TNCR dummy_err
    TCL_Write_Int $target(handle) 0 $AT91C_SPI_RNPR dummy_err
    TCL_Write_Int $target(handle) 0 $AT91C_SPI_RNCR dummy_err
    TCL_Write_Int $target(handle) 0 $AT91C_SPI_TPR dummy_err
    TCL_Write_Int $target(handle) 0 $AT91C_SPI_TCR dummy_err
    TCL_Write_Int $target(handle) 0 $AT91C_SPI_RPR dummy_err
    TCL_Write_Int $target(handle) 0 $AT91C_SPI_RCR dummy_err
    
    puts "-I- Enable the RX and TX PDC transfer requests"
    TCL_Write_Int $target(handle) [expr $AT91C_PDC_RXTEN | $AT91C_PDC_TXTEN] $AT91C_SPI_PTCR dummy_err
    
    puts "-I- End of Init_DataFlash"
}

################################################################################
#  proc DATAFLASH::SelectDataflash
################################################################################
proc DATAFLASH::SelectDataflash { Dataflash_CS } {
    
    # SPI0
    global AT91C_SPI_MR AT91C_SPI_PCS
    global AT91C_SPI_CR AT91C_SPI_SPIEN
    
    variable AT91C_DATAFLASH_TIMEOUT
    variable AT91C_SPI0_PCS0_DATAFLASH
    variable AT91C_SPI0_PCS1_DATAFLASH
    variable ArrayDeviceID
    variable deviceAT45
    variable DataflashDesc
    variable MemorySize
    variable DATAFLASH_initialized

    global target
    set dummy_err 0

    TCL_Write_Int $target(handle) 0xCAFEDECA 0x20000000 dummy_err
    set sdram_read [format "0x%X" [TCL_Read_Int $target(handle) 0x20000000 dummy_err]]
    if {$sdram_read != 0xCAFEDECA} {
        puts "-I- Initialize SDRAM"
        SDRAM::initSDRAM
    }

    DATAFLASH::Init
    
    puts "-I- Switch to the correct PCS of SPI0 Mode Register (Fixed Peripheral Selected)"
    TCL_Write_Int $target(handle) [expr [TCL_Read_Int $target(handle) $AT91C_SPI_MR dummy_err] & 0xFFF0FFFF] $AT91C_SPI_MR dummy_err
    
    switch $Dataflash_CS {
        "AT91C_SPI0_CS0"   {TCL_Write_Int $target(handle) [expr [TCL_Read_Int $target(handle) $AT91C_SPI_MR dummy_err] | [expr $AT91C_SPI_PCS & [expr $AT91C_SPI0_PCS0_DATAFLASH << 16]]] $AT91C_SPI_MR dummy_err
            puts "-I- Chip Select 0 Selected"}
        default            {puts stderr "-E- This Chip Select does not support Dataflash"
            set DATAFLASH_initialized 0
            return -1}
    }
    
    puts "-I- Enable the SPI0"
    TCL_Write_Int $target(handle) $AT91C_SPI_SPIEN $AT91C_SPI_CR dummy_err
    
    puts "-I- Wait for dataflash ready (bit7 of the status register)"
    set returnValue [DATAFLASH::WaitReady $AT91C_DATAFLASH_TIMEOUT]
    
    if {! $returnValue} {
        set DATAFLASH_initialized 0
        puts stderr "-E- DataFlash TimeOut"
        return -1
    }
    if {$DataflashDesc(DataFlash_state) == 0xFF} {
        set DATAFLASH_initialized 0
        puts stderr "-E- DataFlash Not Detected"
        return -1
    }
    
    set dataflashID [expr $DataflashDesc(DataFlash_state) & 0x3C]
    set dataflashID [format "0x%02X" [expr $DataflashDesc(DataFlash_state) & 0x3C]]
    
    switch $ArrayDeviceID($dataflashID) {
        AT45DB011  {set deviceAT45(pages_number) 512
            set deviceAT45(pages_size) 264
            set deviceAT45(page_offset) 9
            set deviceAT45(byte_mask) 0x100
            puts "-I- DataFlash AT45DB011"}
        
        AT45DB021  {set deviceAT45(pages_number) 1024
            set deviceAT45(pages_size) 264
            set deviceAT45(page_offset) 9
            set deviceAT45(byte_mask) 0x100
            puts "-I- DataFlash AT45DB021"}
        
        AT45DB041  {set deviceAT45(pages_number) 2048
            set deviceAT45(pages_size) 264
            set deviceAT45(page_offset) 9
            set deviceAT45(byte_mask) 0x100
            puts "-I- DataFlash AT45DB041"}
        
        AT45DB081  {set deviceAT45(pages_number) 4096
            set deviceAT45(pages_size) 264
            set deviceAT45(page_offset) 9
            set deviceAT45(byte_mask) 0x100
            puts "-I- DataFlash AT45DB081"}
        
        AT45DB161 {set deviceAT45(pages_number) 4096
            set deviceAT45(pages_size) 528
            set deviceAT45(page_offset) 10
            set deviceAT45(byte_mask) 0x300
            puts "-I- DataFlash AT45DB161"}
        
        AT45DB321 {set deviceAT45(pages_number) 8192
            set deviceAT45(pages_size) 528
            set deviceAT45(page_offset) 10
            set deviceAT45(byte_mask) 0x300
            puts "-I- DataFlash AT45DB321"}
        
        AT45DB642  {set deviceAT45(pages_number) 8192
            set deviceAT45(pages_size) 1056
            set deviceAT45(page_offset) 11
            set deviceAT45(byte_mask) 0x700
            puts "-I- DataFlash AT45DB642"}
        
        AT45DB1282 {set deviceAT45(pages_number) 16384
            set deviceAT45(pages_size) 1056
            set deviceAT45(page_offset) 11
            set deviceAT45(byte_mask) 0x700
            puts "-I- DataFlash AT45DB1282"}
        
        AT45DB2562 {set deviceAT45(pages_number) 16384
            set deviceAT45(pages_size) 2112
            set deviceAT45(page_offset) 12
            set deviceAT45(byte_mask) 0xF00
            puts "-I- DataFlash AT45DB2562"}
        
        AT45DB5122 {set deviceAT45(pages_number) 32768
            set deviceAT45(pages_size) 2112
            set deviceAT45(page_offset) 12
            set deviceAT45(byte_mask) 0xF00
            puts "-I- DataFlash AT45DB5122"}

        default {puts stderr "-E- DataFlash Not Supported"
            set DATAFLASH_initialized 0
            return -1}
    }
    
    # Necessary For sendFile and receiveFile methods
    set MemorySize [expr $deviceAT45(pages_number) * $deviceAT45(pages_size)]
    set DATAFLASH_initialized 1
    puts "-I- End of Select DataFlash"
}

################################################################################
#  proc DATAFLASH::EraseAllDataFlash
################################################################################
proc DATAFLASH::EraseAllDataFlash {  } {

    variable DATAFLASH_initialized
    if {$DATAFLASH_initialized != 1} {
        puts stderr "-E- DATAFLASH not initialized"
        return
    }

    variable deviceAT45
    variable AT91C_DATAFLASH_TIMEOUT
    global commandLineMode
    
    # Wait window for loading
    if {$commandLineMode == 0} {
        waitWindows::createWindow "Erase DataFlash" "erase.gif"
        tkwait visibility .topWaitWindow
    }

    global softwareStatusLabelVariable
    set softwareStatusLabelVariable "Erasing whole DataFlash ..."
    
    puts "-I- Erasing whole DataFlash ... Please Wait..."

    set blocknb [expr $deviceAT45(pages_number) / 8]
    while {$blocknb} {
        if { [catch {DATAFLASH::BlockErase [expr $blocknb - 1]} errInfo] } {
            switch $errInfo {
                "AT91C_ERR3" {puts stderr "-E- Invalid DataFlash Block"}
                default {puts stderr "-E- Can't send data, error in connection"}
            }
            if {$commandLineMode == 0} {
                catch {waitWindows::destroyWindow}
            }
            return -1
        }
        
        set returnValue [DATAFLASH::WaitReady $AT91C_DATAFLASH_TIMEOUT]
        set blocknb [expr $blocknb - 1]
    }
    
    if {$commandLineMode == 0} {
        waitWindows::destroyWindow
    }
    set returnValue [DATAFLASH::WaitReady $AT91C_DATAFLASH_TIMEOUT]
    set softwareStatusLabelVariable ""
}

################################################################################
#  proc DATAFLASH::sendFile
################################################################################
proc DATAFLASH::sendFile { name addr } {

    variable DATAFLASH_initialized
    if {$DATAFLASH_initialized != 1} {
        puts stderr "-E- DATAFLASH not initialized"
        return
    }

    if { [catch {set f [open $name r]}] } {
        
        puts stderr "-E- Can't open file $name"
        return
    }
    
    fconfigure $f -translation binary
    
    set size [file size $name]
    puts "-I- File size = $size byte(s)"
    
    if { [catch {DATAFLASH::FastWrite $addr $size $f} errInfo] } {
        switch $errInfo {
            "AT91C_ERR1" {puts stderr "-E- Negative Address or Size"}
            "AT91C_ERR2" {puts stderr "-E- Memory Overflow"}
            default {puts stderr "-E- Can't send data, error in connection"}
        }
        close $f
        return
    }
    close $f
}

################################################################################
#  proc DATAFLASH::receiveFile
################################################################################
proc DATAFLASH::receiveFile {name addr size} {

    variable DATAFLASH_initialized
    if {$DATAFLASH_initialized != 1} {
        puts stderr "-E- DATAFLASH not initialized"
        return
    }

    # put data in a file
    if { [catch {set f2 [open $name w+]}] } {
        puts stderr "-E- Can't open file $name"
        return -1
    }
    fconfigure $f2 -translation binary
    
    #read data from target
    if { [catch {DATAFLASH::FastRead $addr $size $f2} errInfo] } {
        switch $errInfo {
        puts $errInfo
            "AT91C_ERR1" {puts stderr "-E- Dataflash Negative Address or Size"}
            "AT91C_ERR2" {puts stderr "-E- Memory Dataflash Overflow"}
            default {puts stderr "-E- Can't send data, error in connection"}
        }
        close $f2
        return -1
    }
    
    close $f2
}
