#  ----------------------------------------------------------------------------
#          ATMEL Microcontroller Software Support  -  ROUSSET  -
#  ----------------------------------------------------------------------------
#  Copyright (c) 2006, Atmel Corporation
#
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#  this list of conditions and the disclaiimer below.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#  this list of conditions and the disclaimer below in the documentation and/or
#  other materials provided with the distribution. 
#
#  Atmel's name may not be used to endorse or promote products derived from
#  this software without specific prior written permission. 
#
#  DISCLAIMER: THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
#  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
#  DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
#  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
#  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#  ----------------------------------------------------------------------------


################################################################################
################################################################################
## NAMESPACE SDRAM
################################################################################
################################################################################
namespace eval SDRAM {
    variable AT91C_EBI_SDRAM	 [expr 0x20000000]
    variable StartAddress        [expr 0x20000000]
    variable MemorySize          [expr 32*1024*1024]
    
    variable AT91C_SDRC_TWR_2    [expr 0x2 <<  7]
    variable AT91C_SDRC_TRC_4    [expr 0x4 << 11]
    variable AT91C_SDRC_TRP_4    [expr 0x4 << 15]
    variable AT91C_SDRC_TRCD_2   [expr 0x2 << 19]
    variable AT91C_SDRC_TRAS_3   [expr 0x3 << 23]
    variable AT91C_SDRC_TXSR_4   [expr 0x4 << 27]
}

################################################################################
#  proc SDRAM::initSDRAM
################################################################################
proc SDRAM::initSDRAM {} {
          
    global AT91C_EBI_CSA AT91C_EBI_CS1A_SDRAMC
    variable AT91C_EBI_SDRAM
    
    global AT91C_SDRC_CR AT91C_SDRC_NC_9 AT91C_SDRC_NR_13 AT91C_SDRC_CAS_2 AT91C_SDRC_NB_4_BANKS
    variable AT91C_SDRC_TWR_2
    variable AT91C_SDRC_TRC_4
    variable AT91C_SDRC_TRP_4
    variable AT91C_SDRC_TRCD_2
    variable AT91C_SDRC_TRAS_3
    variable AT91C_SDRC_TXSR_4

    global AT91C_SDRC_MR AT91C_SDRC_DBW_16_BITS AT91C_SDRC_MODE_NORMAL_CMD AT91C_SDRC_MODE_NOP_CMD AT91C_SDRC_MODE_PRCGALL_CMD AT91C_SDRC_MODE_RFSH_CMD AT91C_SDRC_MODE_LMR_CMD
    global AT91C_SDRC_TR AT91C_SDRC_COUNT
    
    
    global AT91C_PA23_NWR1_NBS1_CFIOR_NUB AT91C_PA24_SDA10 AT91C_PA25_SDCKE AT91C_PA26_NCS1_SDCS AT91C_PA27_SDWE AT91C_PA28_CAS AT91C_PA29_RAS
    
    global AT91C_PIOA_ASR AT91C_PIOA_BSR AT91C_PIOA_PDR
    global AT91C_PIOB_ASR AT91C_PIOB_BSR AT91C_PIOB_PDR
    global AT91C_PIOC_ASR AT91C_PIOC_BSR AT91C_PIOC_PDR

    global target
    set dummy_err 0

    puts "-I- Configure PIOA as peripheral PIOA23-29"
    # PIOA23 to PIOA29
    set register_value [expr $AT91C_PA23_NWR1_NBS1_CFIOR_NUB | $AT91C_PA24_SDA10 | $AT91C_PA25_SDCKE | $AT91C_PA26_NCS1_SDCS | $AT91C_PA27_SDWE | $AT91C_PA28_CAS | $AT91C_PA29_RAS]
    # AT91F_PIO_CfgPeriph
    TCL_Write_Int $target(handle) 0 $AT91C_PIOA_ASR dummy_err
    TCL_Write_Int $target(handle) $register_value $AT91C_PIOA_BSR dummy_err
    # Set in Periph mode
    TCL_Write_Int $target(handle) [expr $register_value | 0]    $AT91C_PIOA_PDR dummy_err
    
    puts "-I- Configure PIOB as peripheral (A0/A17)"
    # PIOB0 to PIOB17
    set register_value 0x0003FFFF
    # AT91F_PIO_CfgPeriph
    TCL_Write_Int $target(handle) 0 $AT91C_PIOB_ASR dummy_err
    TCL_Write_Int $target(handle) $register_value $AT91C_PIOB_BSR dummy_err
    # Set in Periph mode
    TCL_Write_Int $target(handle) [expr $register_value | 0] $AT91C_PIOB_PDR dummy_err

    puts "-I- Configure PIOC as peripheral (D0/D15)"
    # PIOC0 to PIOC15
    set register_value 0x0000FFFF
    # AT91F_PIO_CfgPeriph
    TCL_Write_Int $target(handle) $register_value $AT91C_PIOC_ASR dummy_err
    TCL_Write_Int $target(handle) 0 $AT91C_PIOC_BSR dummy_err
    # Set in Periph mode
    TCL_Write_Int $target(handle) [expr $register_value | 0] $AT91C_PIOC_PDR dummy_err

    puts "-I- Init the EBI for SDRAM"
    TCL_Write_Int $target(handle) [expr [TCL_Read_Int $target(handle) $AT91C_EBI_CSA dummy_err] | $AT91C_EBI_CS1A_SDRAMC ] $AT91C_EBI_CSA dummy_err

    puts "-I- Init SDRAM"
    puts "-I- 1. A minimum pause of 200us is provided to precede any signal toggle"
    set register_value  [expr $AT91C_SDRC_CR | $AT91C_SDRC_NC_9 | $AT91C_SDRC_NR_13 | $AT91C_SDRC_CAS_2 | $AT91C_SDRC_NB_4_BANKS]
    set register_value  [expr $register_value | $AT91C_SDRC_TWR_2 | $AT91C_SDRC_TRC_4 | $AT91C_SDRC_TRP_4 | $AT91C_SDRC_TRCD_2 | $AT91C_SDRC_TRAS_3 | $AT91C_SDRC_TXSR_4]
    TCL_Write_Int $target(handle) $register_value $AT91C_SDRC_CR dummy_err
    
    puts "-I- 1. SDRAM Initialization Step"
    TCL_Write_Int $target(handle) [expr $AT91C_SDRC_DBW_16_BITS | $AT91C_SDRC_MODE_NOP_CMD] $AT91C_SDRC_MR dummy_err
    puts "-I- *pSDRAM = 0;"
    TCL_Write_Int $target(handle) 0 $AT91C_EBI_SDRAM dummy_err

    puts "-I- 2. A Precharge All command is issued to the SDRAM"
    TCL_Write_Int $target(handle) [expr $AT91C_SDRC_DBW_16_BITS | $AT91C_SDRC_MODE_PRCGALL_CMD] $AT91C_SDRC_MR dummy_err
    puts "-I- *pSDRAM = 0;"
    TCL_Write_Int $target(handle) 0 $AT91C_EBI_SDRAM dummy_err

    puts "-I- 3. Eight Auto-refresh are provided"
    TCL_Write_Int $target(handle) [expr $AT91C_SDRC_DBW_16_BITS | $AT91C_SDRC_MODE_RFSH_CMD] $AT91C_SDRC_MR dummy_err
    for {set i 0} {$i<8} {incr i} {
        puts "-I- *pSDRAM = 0;"
        TCL_Write_Int $target(handle) 0 $AT91C_EBI_SDRAM dummy_err
    }

    puts "-I- 4. A mode register cycle is issued to program the SDRAM parameters"
    TCL_Write_Int $target(handle) [expr $AT91C_SDRC_DBW_16_BITS | $AT91C_SDRC_MODE_LMR_CMD] $AT91C_SDRC_MR dummy_err
    puts "-I- *(pSDRAM+0x20) = 0;"
    TCL_Write_Int $target(handle) 0xcafedede [expr $AT91C_EBI_SDRAM + 0x20] dummy_err

    puts "-I- 5. Write refresh rate into SDRAMC refresh timer COUNT register"
    TCL_Write_Int $target(handle) [expr $AT91C_SDRC_COUNT & 0x180] $AT91C_SDRC_TR dummy_err

    puts "-I- 6. A Normal Mode Command is provided, 3 clocks after tMRD is set"
    TCL_Write_Int $target(handle) [expr $AT91C_SDRC_DBW_16_BITS | $AT91C_SDRC_MODE_NORMAL_CMD] $AT91C_SDRC_MR dummy_err
    puts "-I- *pSDRAM = 0;"
    TCL_Write_Int $target(handle) 0 $AT91C_EBI_SDRAM dummy_err

    puts "-I- End of Init_SDRAM"
}

################################################################################
#  proc SDRAM::sendFile
################################################################################
proc SDRAM::sendFile { name addr } {
    
    global valueOfDataForSendFile
    if { [catch {set f [open $name r]}] } {
        set valueOfDataForSendFile 0
        puts stderr "-E- Can't open file $name"
        return -1
    }
    
    fconfigure $f -translation binary
    
    set size [file size $name]
    puts "-I- File size = $size byte(s)"
    set valueOfDataForSendFile [read $f $size]
    
    close $f

    global target
    set dummy_err 0
    if { [catch {TCL_Write_Data $target(handle) $addr valueOfDataForSendFile $size dummy_err}] } {
        puts stderr "-E- Can't send data, error in connection"
        set valueOfDataForSendFile 0
        return
    }
    
    set valueOfDataForSendFile 0
}

################################################################################
#  proc SDRAM::receiveFile
################################################################################
proc SDRAM::receiveFile {name addr size} {
    #read data from target
    global target
    set dummy_err 0
    if { [catch {set result [TCL_Read_Data $target(handle) $addr $size dummy_err]}] } {
        puts stderr "-E- Can't read data, error in connection"
        return -1
    }
    # put data in a file
    if { [catch {set f2 [open $name w+]}] } {
        puts stderr "-E- Can't open file $name"
        return -1
    }
    fconfigure $f2 -translation binary
    puts -nonewline $f2 $result
    close $f2
}
