#  ----------------------------------------------------------------------------
#          ATMEL Microcontroller Software Support  -  ROUSSET  -
#  ----------------------------------------------------------------------------
#  Copyright (c) 2006, Atmel Corporation
#
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#  this list of conditions and the disclaiimer below.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#  this list of conditions and the disclaimer below in the documentation and/or
#  other materials provided with the distribution. 
#
#  Atmel's name may not be used to endorse or promote products derived from
#  this software without specific prior written permission. 
#
#  DISCLAIMER: THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
#  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
#  DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
#  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
#  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#  ----------------------------------------------------------------------------


################################################################################
################################################################################
## NAMESPACE FLASH
################################################################################
################################################################################
namespace eval FLASH {
    
    # FLASH Features fl020
    variable AT91C_NB_OF_PAGES            1024
    variable AT91C_NB_OF_SECTORS          16
    variable AT91C_NB_OF_PAGES_BY_SECTOR  64
    variable AT91C_PAGE_SIZE              256
    variable AT91C_SECTOR_SIZE            [expr $AT91C_NB_OF_PAGES_BY_SECTOR * $AT91C_PAGE_SIZE]
    variable AT91C_FLASH_SIZE             [expr $AT91C_NB_OF_PAGES * $AT91C_PAGE_SIZE]
    variable AT91C_ADDR_MASK              [expr (2 * $AT91C_FLASH_SIZE) - 1]
    variable AT91C_PAGE_SHIFT             8
    variable AT91C_SECTOR_SHIFT           16
    variable AT91C_SECTOR_MASK            0xFFFF0000
    variable AT91C_MC_CORRECT_KEY         [expr 0x5A << 24]
    variable AT91C_FLASH_TIMEOUT          1000000
    variable baseAddressFLASH             [expr 0x100000]
    variable MemorySize                   [expr 512*1024]

    # Page Buffer & Page Number Address used by the C binary project
    variable comType                      0x2013FC
    variable pageBuf                      0x201400
    variable pageNb                       [expr $pageBuf + $AT91C_PAGE_SIZE]
    variable EFCx                         [expr $pageNb + 4]
 
    # Link address & Binary name in the C project
    variable monitorAddr                  0x201600
    variable monitorName                  "SAM-BA-Flash.bin"
    
    variable ForceUnlockBeforeWrite       0
    variable ForceLockAfterWrite          0
}

global target
set dummy_err 0
if {$target(comType) == 1} {
    # Retrieve MCK through DBGU baudrate programmation
    global AT91C_DBGU_BRGR
    set fmcn [expr [TCL_Read_Int $target(handle) $AT91C_DBGU_BRGR dummy_err] & 0xFFFF]
    set fmcn [expr 16 * 115200 * $fmcn]
    set fmcn [expr $fmcn / 1000000]
    set GPNVM_FMR [expr $fmcn]
    set GPNVM_FMR [expr 0x00000100 | [expr $GPNVM_FMR << 16]]
    set PROG_FMR [expr $fmcn *3/2]
    set PROG_FMR [expr 0x00000100 | [expr $PROG_FMR << 16]]
} else  {
    # Set GPNVM_FMR configuration for GPNVM bits operations
    set GPNVM_FMR 0x00480200
    # Set PROG_FMR configuration for Programming operations
    set PROG_FMR 0x00480200
}


################################################################################
#  proc FLASH::OFFSET_PAGE
################################################################################
proc FLASH::OFFSET_PAGE { X } {
    variable AT91C_PAGE_SIZE
    
    return [expr $X * $AT91C_PAGE_SIZE]
}

################################################################################
#  proc FLASH::PerformCmd
################################################################################
proc FLASH::PerformCmd { transfer_cmd efc } {
    global AT91C_EFC0_FCR AT91C_EFC1_FCR
    
    global target
    set dummy_err 0
    variable AT91C_FLASH_TIMEOUT

    if {$efc == 0} {
        set fcr $AT91C_EFC0_FCR
    } else  {
        set fcr $AT91C_EFC1_FCR
    }
    TCL_Write_Int $target(handle) $transfer_cmd $fcr dummy_err

    # Wait for FLASH Ready
    return [FLASH::WaitReady $AT91C_FLASH_TIMEOUT $efc]
}

################################################################################
#  proc FLASH::CfgModeRegister
################################################################################
proc FLASH::CfgModeRegister { mode efc } {
    global AT91C_EFC0_FMR AT91C_EFC1_FMR
    variable AT91C_FLASH_TIMEOUT

    global target
    set dummy_err 0

    if {$efc == 0} {
        set fmr $AT91C_EFC0_FMR
    } else  {
        set fmr $AT91C_EFC1_FMR
    }
    
    # Write to the FMR register
    TCL_Write_Int $target(handle) $mode $fmr dummy_err
    # Wait for FLASH Ready
    return [FLASH::WaitReady $AT91C_FLASH_TIMEOUT $efc]
}

################################################################################
#  proc FLASH::WaitReady
################################################################################
proc FLASH::WaitReady { timeout efc } {
    global AT91C_EFC0_FSR AT91C_EFC1_FSR AT91C_MC_FRDY

    global target
    set dummy_err 0

    if {$efc == 0} {
        set fsr $AT91C_EFC0_FSR
    } else  {
        set fsr $AT91C_EFC1_FSR
    }
    
    set Status 0
    while {$Status == 0} {
        set timeout [expr $timeout - 1]
        if {$timeout == 0} {
            puts stderr "-E- TimeOut error"
            return -1
        }
        set Status [expr [TCL_Read_Int $target(handle) $fsr dummy_err] & $AT91C_MC_FRDY ]
    }
    return 1
}

################################################################################
#  proc FLASH::ProgramPage
################################################################################
proc FLASH::ProgramPage { page File efc } {
    global valueOfDataForSendFile

    variable AT91C_PAGE_SIZE

    variable comType
    variable monitorAddr
    variable pageNb
    variable pageBuf
    variable EFCx        
    
    global target
    set dummy_err 0

    TCL_Write_Int $target(handle) $target(comType) $comType dummy_err    
    # Copy corresponding page into SRAM reserved memory (at address $pageNb)
    TCL_Write_Int $target(handle) $page $pageNb dummy_err
    # Copy EFC value into SRAM reserved memory (at address $EFCx)
    TCL_Write_Int $target(handle) $efc $EFCx dummy_err
            
    set valueOfDataForSendFile [read $File $AT91C_PAGE_SIZE]
    # Copy corresponding page content into SRAM reserved memory (at address $pageBuf)
    catch {TCL_Write_Data $target(handle) $pageBuf valueOfDataForSendFile $AT91C_PAGE_SIZE dummy_err}
    # Branch on C binary
    catch {TCL_Go $target(handle) $monitorAddr dummy_err}    
}

################################################################################
#  proc FLASH::SetGP
################################################################################
proc FLASH::SetGP { gp } {    
    global AT91C_MC_FCMD_SET_GP_NVM AT91C_MC_PAGEN
    variable AT91C_MC_CORRECT_KEY
    variable    AT91C_PAGE_SHIFT
    global GPNVM_FMR
    
    # EFC0 only
    set efc 0
    
    # Restore configuration for GPNVM bits operations
    FLASH::CfgModeRegister $GPNVM_FMR $efc
    
    # Send Page Program Command
    set cmd [expr $AT91C_MC_PAGEN & [expr $gp << $AT91C_PAGE_SHIFT]]
    FLASH::PerformCmd [expr $AT91C_MC_CORRECT_KEY | $AT91C_MC_FCMD_SET_GP_NVM | $cmd] $efc
}

################################################################################
#  proc FLASH::ClearGP
################################################################################
proc FLASH::ClearGP { gp } {
    global AT91C_MC_FCMD_CLR_GP_NVM AT91C_MC_PAGEN
    variable AT91C_MC_CORRECT_KEY
    variable    AT91C_PAGE_SHIFT
    global GPNVM_FMR
    
    # EFC0 only
    set efc 0
        
    # Restore configuration for GPNVM bits operations
    FLASH::CfgModeRegister $GPNVM_FMR $efc
    
    # Send Page Program Command
    set cmd [expr $AT91C_MC_PAGEN & [expr $gp << $AT91C_PAGE_SHIFT]]
    FLASH::PerformCmd [expr $AT91C_MC_CORRECT_KEY | $AT91C_MC_FCMD_CLR_GP_NVM | $cmd] $efc
}

################################################################################
#  proc FLASH::ScriptGPNMV
################################################################################
proc FLASH::ScriptGPNMV { index } {   
    switch $index {
        0 {
            if { [catch {FLASH::SetGP 0}] } {
                puts stderr "-E- Enable BrownOut failed"
                return -1
            } else  {puts "-I- BrownOut Enabled"}
        }
        1 {
            if { [catch {FLASH::ClearGP 0}] } {
                puts stderr "-E- Disable BrownOut failed"
                return -1
            } else  {puts "-I- BrownOut Disabled"}
        }
        2 {
            if { [catch {FLASH::SetGP 1}] } {
                puts stderr "-E- Enable BrownOut Reset failed"
                return -1
            } else  {puts "-I- BrownOut Reset Enabled"}
        }
        3 {
            if { [catch {FLASH::ClearGP 1}] } {
                puts stderr "-E- Disable BrownOut Reset failed"
                return -1
            } else  {puts "-I- BrownOut Reset Disabled"}
        }
        4 {
            if { [catch {FLASH::SetGP 2}] } {
                puts stderr "-E- Boot from Flash failed"
                return -1
            } else  {puts "-I- Boot from Flash selected"}
        }
        5 {
            if { [catch {FLASH::ClearGP 2}] } {
                puts stderr "-E- Boot from ROM failed"
                return -1
            } else  {puts "-I- Boot from ROM selected"}
        }
    }
}

################################################################################
#  proc FLASH::SetSecurityBit
################################################################################
proc FLASH::SetSecurityBit { } {
    global AT91C_MC_FCMD_SET_SECURITY
    variable AT91C_MC_CORRECT_KEY
    global GPNVM_FMR
    
    # EFC0 only
    set efc 0
    
    # Restore configuration for GPNVM bits operations
    FLASH::CfgModeRegister $GPNVM_FMR $efc
    
    # Send Page Program Command
    FLASH::PerformCmd [expr $AT91C_MC_CORRECT_KEY | $AT91C_MC_FCMD_SET_SECURITY] $efc
}

################################################################################
#  proc FLASH::ScriptSetSecurityBit
################################################################################
proc FLASH::ScriptSetSecurityBit { } {

    if { [catch {FLASH::SetSecurityBit}] } {
        puts stderr "-E- Set Security Bit failed"
        return -1
    }
    puts "-I- Security Bit Set"
}

################################################################################
#  proc FLASH::LockSector
################################################################################
proc FLASH::LockSector { PageInSector efc } {
    global AT91C_MC_FCMD_LOCK AT91C_MC_PAGEN
    variable AT91C_MC_CORRECT_KEY
    variable AT91C_PAGE_SHIFT
    global GPNVM_FMR
    
    # Restore configuration for GPNVM bits operations
    FLASH::CfgModeRegister $GPNVM_FMR $efc
    
    # Send Lock Command
    set cmd [expr $AT91C_MC_PAGEN & [expr $PageInSector << $AT91C_PAGE_SHIFT]]
    FLASH::PerformCmd [expr $AT91C_MC_CORRECT_KEY | $AT91C_MC_FCMD_LOCK | $cmd] $efc
}

################################################################################
#  proc FLASH::AskForLockSector
################################################################################
proc FLASH::AskForLockSector { dest sizeToWrite } {
    variable AT91C_NB_OF_PAGES_BY_SECTOR
    variable AT91C_FLASH_TIMEOUT
    variable AT91C_NB_OF_SECTORS
    variable AT91C_SECTOR_SIZE
    global   PROG_FMR
    variable    ForceLockAfterWrite
    global      commandLineMode
    
    # Compute first and last lock regions
    set first_lockregion [expr $dest / $AT91C_SECTOR_SIZE]
    set last_lockregion [expr [expr $dest + $sizeToWrite] / $AT91C_SECTOR_SIZE]
    if { [expr [expr $dest + $sizeToWrite] % $AT91C_SECTOR_SIZE] == 0} {
        set last_sector [expr $last_lockregion - 1]
    }
    
    if {$commandLineMode == 0} {
        set returnValue [messageDialg warning.gif "Do you want to lock involved lock region(s) ($first_lockregion to $last_lockregion) ?" "Lock region(s) to lock" yesno ]
        if {$returnValue} {
            return -1
        } else {
            set ForceLockAfterWrite 1
        }
    }

    if {$ForceLockAfterWrite} {
        # Lock all sectors involved in the write
        for {set i $first_lockregion} {$i <= $last_lockregion } {incr i} {
            if {$i < $AT91C_NB_OF_SECTORS} {         
                set efc 0
            } else  {
                set efc 1
            }
            FLASH::LockSector [expr $i * $AT91C_NB_OF_PAGES_BY_SECTOR] $efc
            set returnValue [FLASH::WaitReady $AT91C_FLASH_TIMEOUT $efc]        
            puts "-I- Sector $i locked"
        }
    }
    
    # Restore configuration for programming operations
    FLASH::CfgModeRegister $PROG_FMR 0
    FLASH::CfgModeRegister $PROG_FMR 1
    
    return 1
}

################################################################################
#  proc FLASH::UnlockSector
################################################################################
proc FLASH::UnlockSector { PageInSector efc } {
    global AT91C_MC_FCMD_UNLOCK AT91C_MC_PAGEN
    variable AT91C_MC_CORRECT_KEY
    variable    AT91C_PAGE_SHIFT
    global GPNVM_FMR
    
    # Restore configuration for GPNVM bits operations
    FLASH::CfgModeRegister $GPNVM_FMR $efc
    
    # Send Unlock Command
    set cmd [expr $AT91C_MC_PAGEN & [expr $PageInSector << $AT91C_PAGE_SHIFT]]
    FLASH::PerformCmd [expr $AT91C_MC_CORRECT_KEY | $AT91C_MC_FCMD_UNLOCK | $cmd] $efc
}

################################################################################
#  proc FLASH::IsSectorLocked
################################################################################
proc FLASH::IsSectorLocked { lockregion efc } {
    global AT91C_EFC0_FSR AT91C_EFC1_FSR
    variable    AT91C_SECTOR_SHIFT
    variable    AT91C_SECTOR_MASK

    global target
    set dummy_err 0

    if {$efc == 0} {
        set fsr $AT91C_EFC0_FSR
    } else  {
        set fsr $AT91C_EFC1_FSR
    }
    
    set Status [expr [TCL_Read_Int $target(handle) $fsr dummy_err] & $AT91C_SECTOR_MASK]
    set Status [expr $Status & [expr 1 << [expr $lockregion + $AT91C_SECTOR_SHIFT] ] ]
    
    if {$Status== 0} {
        return 0
    }

    return 1
}

################################################################################
#  proc FLASH::AskForUnlockSector
################################################################################
proc FLASH::AskForUnlockSector { dest sizeToWrite } {
    variable    AT91C_NB_OF_PAGES_BY_SECTOR
    variable    AT91C_FLASH_TIMEOUT
    variable    AT91C_NB_OF_SECTORS
    variable    AT91C_SECTOR_SIZE
    global      PROG_FMR
    variable    ForceUnlockBeforeWrite
    global      commandLineMode
    
    # Compute first and last lock regions
    set first_lockregion [expr $dest / $AT91C_SECTOR_SIZE]
    set last_lockregion [expr [expr $dest + $sizeToWrite] / $AT91C_SECTOR_SIZE]
    if { [expr [expr $dest + $sizeToWrite] % $AT91C_SECTOR_SIZE] == 0} {
        set last_sector [expr $last_lockregion - 1]
    }
    
    # Test if at least one sector involved in the write is locked
    set OneSectorLocked 0
    for {set i $first_lockregion} {$i <= $last_lockregion } {incr i} {
        if {$i < $AT91C_NB_OF_SECTORS} {
            set efc 0
        } else  {
            set efc 1
        }
        set returnValue [FLASH::IsSectorLocked $i $efc]
        if {$returnValue} {
            set OneSectorLocked 1
        }
    }
    
    # Ask for Unlock Sector(s)
    if {$OneSectorLocked} {
        if {$commandLineMode == 0} {
            set returnValue [messageDialg warning.gif "Do you want to unlock involved lock region(s) ($first_lockregion to $last_lockregion) ?" "At least one lock region is locked !" yesno ]
            if {$returnValue} {
                return -1
            } else {
                set ForceUnlockBeforeWrite 1
            }
        }
    } else {
        return 1
    }

    if {$ForceUnlockBeforeWrite} {
        # Unlock all sectors involved in the write
        for {set i $first_lockregion} {$i <= $last_lockregion } {incr i} {
            if {$i < $AT91C_NB_OF_SECTORS} {
                set efc 0
            } else  {
                set efc 1
            }
            FLASH::UnlockSector [expr $i * $AT91C_NB_OF_PAGES_BY_SECTOR] $efc
            set returnValue [FLASH::WaitReady $AT91C_FLASH_TIMEOUT $efc]
            puts "-I- Sector $i unlocked"
        }
        # Restore configuration for programming operations
        FLASH::CfgModeRegister $PROG_FMR 0
        FLASH::CfgModeRegister $PROG_FMR 1
        return 1
    } else {
        return -1
    }
}

################################################################################
#  proc FLASH::WritePartialPage
################################################################################
proc FLASH::WritePartialPage { File dest sizeToWrite efc } {
    global AT91C_MC_PAGEN AT91C_MC_FCMD_START_PROG
    global valueOfDataForSendFile
    variable AT91C_MC_CORRECT_KEY
    variable AT91C_PAGE_SIZE
    variable AT91C_FLASH_TIMEOUT
    variable AT91C_FLASH_SIZE
    variable AT91C_PAGE_SHIFT
    variable baseAddressFLASH
  
    global target
    set dummy_err 0

    set page [expr $dest / $AT91C_PAGE_SIZE]
    set AdrInPage [expr $dest % $AT91C_PAGE_SIZE]
  
    #read corresponding page from target
    if { [catch {set result [TCL_Read_Data $target(handle) [expr $baseAddressFLASH + [expr $efc * $AT91C_FLASH_SIZE] + [FLASH::OFFSET_PAGE $page]] $AT91C_PAGE_SIZE dummy_err]}] } {
        puts stderr "-E- Can't read data, error in connection"
        return -1
    }

    # put data in a file
    if { [catch {set f [open "PartialPageWrite.tmp" w+]}] } {
        puts stderr "-E- Can't open file $name"
        return -1
    }
    fconfigure $f -translation binary
    puts -nonewline $f $result
    
    # Modify corresponding page
    seek $f $AdrInPage start
    set valueOfDataForSendFile [read $File $sizeToWrite]
    puts -nonewline $f $valueOfDataForSendFile
    seek $f 0 start
    
    # Write Modified File
    set returnValue [FLASH::ProgramPage $page $f $efc]
    
    close $f
    file delete "PartialPageWrite.tmp"
    
    return $returnValue
}

################################################################################
#  proc FLASH::WriteBlock
################################################################################
proc FLASH::WriteBlock { dest sizeToWrite File efc } {
    variable AT91C_NB_OF_PAGES
    variable AT91C_PAGE_SIZE
    variable AT91C_FLASH_TIMEOUT
    variable AT91C_ADDR_MASK
    variable AT91C_FLASH_SIZE
    variable AT91C_SECTOR_SIZE
    variable monitorAddr
    variable pageNb
    variable pageBuf
    variable EFCx
    variable comType
    
    global target
    set dummy_err 0

    if { [expr $dest < 0] || [expr $sizeToWrite < 0] } {
        error AT91C_ERR1
    }
    
    # mask on base flash address
    set dest [expr $dest & $AT91C_ADDR_MASK]
    
    if { [expr $dest + $sizeToWrite] > $AT91C_FLASH_SIZE } {
        error AT91C_ERR2
    }
    
    # 1st step: Address is not modulo AT91C_PAGE_SIZE
    set AdrInPage [expr $dest % $AT91C_PAGE_SIZE]
    if {$AdrInPage} {
        set length [expr $AT91C_PAGE_SIZE - $AdrInPage]
        if {$sizeToWrite < $length} {
            set length $sizeToWrite
        }
        FLASH::WritePartialPage $File $dest $length $efc
        
        set sizeToWrite [expr $sizeToWrite - $length]
        set dest [expr $dest + $length]
    }

    # 2nd step: write whole page(s)
    while { [expr $sizeToWrite - $AT91C_PAGE_SIZE] >= 0 } {
        set page [expr $dest / $AT91C_PAGE_SIZE]

    TCL_Write_Int $target(handle) $target(comType) $comType dummy_err    
        # Copy corresponding page into SRAM reserved memory (at address $pageNb)
        TCL_Write_Int $target(handle) $page $pageNb dummy_err
        # Copy EFC value into SRAM reserved memory (at address $EFCx)
        TCL_Write_Int $target(handle) $efc $EFCx dummy_err

        set valueOfDataForSendFile [read $File $AT91C_PAGE_SIZE]
        # Copy corresponding page content into SRAM reserved memory (at address $pageBuf)
        catch {TCL_Write_Data $target(handle) $pageBuf valueOfDataForSendFile $AT91C_PAGE_SIZE dummy_err}
        # Branch on C binary
        catch {TCL_Go $target(handle) $monitorAddr dummy_err}
        
        set sizeToWrite [expr $sizeToWrite - $AT91C_PAGE_SIZE]
        set dest [expr $dest + $AT91C_PAGE_SIZE]
    }
    
    # 3rd step: write rest of file (size inferior to a page)
    if {$sizeToWrite > 0} {
        set returnValue [FLASH::WaitReady $AT91C_FLASH_TIMEOUT $efc]
        FLASH::WritePartialPage $File $dest $sizeToWrite $efc
    }
        
    return 1
}

################################################################################
#  proc FLASH::Write
################################################################################
proc FLASH::Write { dest sizeToWrite File } {
    global valueOfDataForSendFile
    variable AT91C_FLASH_SIZE
    variable AT91C_ADDR_MASK
    global   PROG_FMR
    variable monitorName
    variable monitorAddr
    variable MemorySize
    
    global libPath
    global target
    set dummy_err 0

    if { [expr $dest < 0] || [expr $sizeToWrite < 0] } {
        error AT91C_ERR1
    }
    
    # mask on base flash address
    set dest [expr $dest & $AT91C_ADDR_MASK]
    
    if { [expr $dest + $sizeToWrite] > $MemorySize } {
        error AT91C_ERR2
    }
    
    # Open monitor file
    set fileName "$libPath(extLib)/$target(board)/$monitorName"
    if { [catch {set f [open $fileName r]}] } {
        puts stderr "-E- Can't open file $fileName"
        return -1
    }
    
    # Restore configuration for programming operations
    FLASH::CfgModeRegister $PROG_FMR 0
    FLASH::CfgModeRegister $PROG_FMR 1

    # Copy $fileName content into SRAM reserved memory (at address $monitorAddr)
    fconfigure $f -translation binary
    set size [file size $fileName]
    set valueOfDataForSendFile [read $f $size]
    catch {TCL_Write_Data $target(handle) $monitorAddr valueOfDataForSendFile $size dummy_err}
    close $f
    
    # Check for locked lock regions in order to unlock them, return if user refuses
    set returnValue [FLASH::AskForUnlockSector $dest $sizeToWrite]
    if {$returnValue == -1} {
        puts stderr "-E- Send file failed: some lock regions are always locked !"
        return -1
    }

    set start_addr [expr $dest / $AT91C_FLASH_SIZE]
    if { [expr $start_addr > 0] } {
        # Program only EFC1 block
        puts "Program EFC1 only"
        set block_offset [expr $dest % $AT91C_FLASH_SIZE]
        puts [format "Block_offset = 0x%x" $block_offset]
        FLASH::WriteBlock $block_offset $sizeToWrite $File 1
    } else  {
        set start_addr [expr $dest + $sizeToWrite]
        if { [expr $start_addr > $AT91C_FLASH_SIZE] } {
            puts "Program both EFC"
            # Program EFC0 block
            set block_offset $dest
            set block_size [expr $AT91C_FLASH_SIZE - $dest]
            puts [format "EFC0 Block_offset = 0x%x" $block_offset]
            puts [format "EFC0 Block_size = 0x%x" $block_size]
            FLASH::WriteBlock $block_offset $block_size $File 0
            
            # Program EFC1 block
            set block_offset 0
            set block_size [expr $sizeToWrite - $block_size]
            puts [format "EFC1 Block_offset = 0x%x" $block_offset]
            puts [format "EFC1 Block_size = 0x%x" $block_size]
            FLASH::WriteBlock $block_offset $block_size $File 1

        } else  {
            # Program only EFC0 block
            puts "Program EFC0 only"
            FLASH::WriteBlock $dest $sizeToWrite $File 0
        }
    }

    # Lock all sectors involved in the write
    set returnValue [FLASH::AskForLockSector $dest $sizeToWrite]
    
    return returnValue
}

################################################################################
#  proc FLASH::sendFile
################################################################################
proc FLASH::sendFile {name addr} {

    if { [catch {set f [open $name r]}] } {
        puts stderr "-E- Can't open file $name"
        return -1
    }
    
    fconfigure $f -translation binary
    
    set size [file size $name]
    puts "-I- File size = $size byte(s)"

    if {[catch {FLASH::Write $addr $size $f} errInfo]} {
        switch $errInfo {
            "AT91C_ERR1" {puts stderr "-E- Negative Address or Size"}
            "AT91C_ERR2" {puts stderr "-E- Memory Overflow"}
            default {puts stderr "-E- Can't send data, error in connection"}
        }
        close $f
        return -1
    }
    
    close $f
}

################################################################################
#  proc FLASH::receiveFile
################################################################################
proc FLASH::receiveFile {name addr size} {
    global target
    set dummy_err 0

    # read data from target
    if { [catch {set result [TCL_Read_Data $target(handle) $addr $size dummy_err]}] } {
        puts stderr "-E- Can't read data, error in connection"
        return -1
    }
    # put data in a file
    if { [catch {set f2 [open $name w+]}] } {
        puts stderr "-E- Can't open file $name"
        return -1
    }
    fconfigure $f2 -translation binary
    puts -nonewline $f2 $result
    close $f2
}

################################################################################
#  proc FLASH::ScriptEraseAllFlash
################################################################################
proc FLASH::ScriptEraseAllFlash { } {
    variable MemorySize
    variable AT91C_NB_OF_SECTORS
    global   PROG_FMR
    global commandLineMode

    # Check for locked lock regions in order to unlock them, return if user refuses
    catch {set returnValue [FLASH::AskForUnlockSector 0 [expr $MemorySize - 1] ]}
    if {$returnValue == -1} {
        puts stderr "-E- Erase All Flash failed: some lock regions are always locked !"
        return -1
    }

    # Wait window for loading
    if {$commandLineMode == 0} {
        waitWindows::createWindow "Erase Flash" "erase.gif"
        tkwait visibility .topWaitWindow
    }

    for {set i 0} {$i < 2 } {incr i} {
        # Restore configuration for programming operations
        FLASH::CfgModeRegister $PROG_FMR $i
    
        if { [catch {FLASH::EraseAllFlash $i}]} {
            puts stderr "-E- Erase Flash Block $i failed"
            if {$commandLineMode == 0} {
                waitWindows::destroyWindow
            }
            return -1
        }
    }

    puts "-I- All Flash erased"

    if {$commandLineMode == 0} {
        waitWindows::destroyWindow
    }
}

################################################################################
#  proc FLASH::EraseAllFlash
################################################################################
proc FLASH::EraseAllFlash { efc } {
    
    global AT91C_MC_FCMD_ERASE_ALL
    variable AT91C_MC_CORRECT_KEY
    
    # Send Page Program Command
    set returnValue [FLASH::PerformCmd [expr $AT91C_MC_CORRECT_KEY | $AT91C_MC_FCMD_ERASE_ALL] $efc]
}
