#  ----------------------------------------------------------------------------
#          ATMEL Microcontroller Software Support  -  ROUSSET  -
#  ----------------------------------------------------------------------------
#  Copyright (c) 2006, Atmel Corporation
#
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#  this list of conditions and the disclaiimer below.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#  this list of conditions and the disclaimer below in the documentation and/or
#  other materials provided with the distribution. 
#
#  Atmel's name may not be used to endorse or promote products derived from
#  this software without specific prior written permission. 
#
#  DISCLAIMER: THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
#  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
#  DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
#  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
#  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#  ----------------------------------------------------------------------------


if { [ catch { source "$libPath(extLib)/$target(board)/SRAM.tcl"} errMsg] } {
    messageDialg  error.gif "Memory Description File not found:\n$errMsg" "File not found" ok
    exit
}

if { [ catch { source "$libPath(extLib)/$target(board)/FLASH.tcl"} errMsg] } {
    messageDialg  error.gif "Memory Description File not found:\n$errMsg" "File not found" ok
    exit
}

if { [ catch { source "$libPath(extLib)/$target(board)/DATAFLASH.tcl"} errMsg] } {
    messageDialg  error.gif "Memory Description File not found:\n$errMsg" "File not found" ok
    exit
}


array set memoryAlgo {
	"SRAM"                    "::at91sam7xc512_sram"
	"Flash"                   "::at91sam7xc512_flash"
	"DataFlash AT45DB/DCB"    "::at91sam7xc512_dataflash"
        "Peripheral"              "::at91sam7xc512_peripheral"
        "ROM"                     "::at91sam7xc512_rom"
        "REMAP"                   "::at91sam7xc512_remap"
}

array set at91sam7xc512_sram {
	dftDisplay  1
        dftDefault  0
	dftAddress  0x200000
	dftSize     0x20000
	dftSend     "SRAM::sendFile"
	dftReceive  "SRAM::receiveFile"
	dftScripts  ""
}

array set at91sam7xc512_flash {
	dftDisplay  1
        dftDefault  1
	dftAddress  0x100000
	dftSize     0x80000
	dftSend     "FLASH::sendFile"
	dftReceive  "FLASH::receiveFile"
	dftScripts  "::at91sam7xc512_flash_scripts"
}

array set at91sam7xc512_flash_scripts {
        "Erase All Flash"                      "FLASH::ScriptEraseAllFlash"
        "Enable BrownOut Detector (GPNVM0)"    "FLASH::ScriptGPNMV 0"
        "Disable BrownOut Detector (GPNVM0)"   "FLASH::ScriptGPNMV 1"
        "Enable BrownOut Reset (GPNVM1)"       "FLASH::ScriptGPNMV 2"
        "Disable BrownOut Reset (GPNVM1)"      "FLASH::ScriptGPNMV 3"
        "Boot from Flash (GPNVM2)"             "FLASH::ScriptGPNMV 4"
        "Boot from ROM (GPNVM2)"               "FLASH::ScriptGPNMV 5"
        "Enable Security Bit"                  "FLASH::ScriptSetSecurityBit"
}

array set at91sam7xc512_dataflash {
	dftDisplay  1
        dftDefault  0
	dftAddress  0x0
	dftSize     "$DATAFLASH::MemorySize"
	dftSend     "DATAFLASH::sendFile"
	dftReceive  "DATAFLASH::receiveFile"
	dftScripts  "::at91sam7xc512_dataflash_scripts"
}

array set at91sam7xc512_dataflash_scripts {
	"Enable Dataflash on CS0"   "DATAFLASH::SelectDataflash AT91C_SPI0_CS0"
	"Enable Dataflash on CS1"   "DATAFLASH::SelectDataflash AT91C_SPI0_CS1"
	"Erase All"                 "DATAFLASH::EraseAllDataFlash"
}

array set at91sam7xc512_peripheral {
	dftDisplay  0
        dftDefault  0
	dftAddress  0xF0000000
	dftSize     0x10000000
	dftSend     ""
	dftReceive  ""
	dftScripts  ""
}

array set at91sam7xc512_rom {
	dftDisplay  0
        dftDefault  0
	dftAddress  0x300000
	dftSize     0x2000
	dftSend     ""
	dftReceive  ""
	dftScripts  ""
}

array set at91sam7xc512_remap {
	dftDisplay  0
        dftDefault  0
	dftAddress  0x00000000
	dftSize     0x2000
	dftSend     ""
	dftReceive  ""
	dftScripts  ""
}
