#  ----------------------------------------------------------------------------
#          ATMEL Microcontroller Software Support  -  ROUSSET  -
#  ----------------------------------------------------------------------------
#  Copyright (c) 2006, Atmel Corporation
#
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#  this list of conditions and the disclaiimer below.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#  this list of conditions and the disclaimer below in the documentation and/or
#  other materials provided with the distribution. 
#
#  Atmel's name may not be used to endorse or promote products derived from
#  this software without specific prior written permission. 
#
#  DISCLAIMER: THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
#  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
#  DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
#  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
#  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#  ----------------------------------------------------------------------------


################################################################################
################################################################################
## NAMESPACE SDRAM
################################################################################
################################################################################
namespace eval SDRAM {
    variable   SDRAM_initialized  [expr 0]
}

################################################################################
#  proc SDRAM::initSDRAM
################################################################################
proc SDRAM::initSDRAM_48 {} {
          
    global AT91C_EBI_SDRAM AT91C_CCFG_EBICSA AT91C_MATRIX_CS1A
    global AT91C_SDRAMC_CR AT91C_SDRAMC_TXSR_3 AT91C_SDRAMC_TRAS_2 AT91C_SDRAMC_TRCD_1 
    global AT91C_SDRAMC_TRC_4 AT91C_SDRAMC_TWR_1 AT91C_SDRAMC_TRP_1
    global AT91C_SDRAMC_CAS_2 AT91C_SDRAMC_DBW_32_BITS AT91C_SDRAMC_NC_9 AT91C_SDRAMC_NR_13 AT91C_SDRAMC_NB_4_BANKS
    global AT91C_SDRAMC_MR AT91C_SDRAMC_MODE_PRCGALL_CMD AT91C_SDRAMC_MODE_RFSH_CMD 
    global AT91C_SDRAMC_MODE_LMR_CMD AT91C_SDRAMC_MODE_NORMAL_CMD
    global AT91C_SDRAMC_TR AT91C_SDRAMC_COUNT
    global AT91C_PIOC_ASR AT91C_PIOC_BSR AT91C_PIOC_PDR

    global target
    set dummy_err 0

    variable SDRAM_initialized
    if {$SDRAM_initialized != 0} {
        puts "-I- SDRAM already initialized"
        return
    }

    puts "-I- Configure PIOC as peripheral (D16/D31)"
    # PIOC16 to 31
    set register_value 0xFFFF0000
    # AT91F_PIO_CfgPeriph
    TCL_Write_Int $target(handle) $register_value $AT91C_PIOC_ASR dummy_err
    TCL_Write_Int $target(handle) 0 $AT91C_PIOC_BSR dummy_err
    # Set in Periph mode
    TCL_Write_Int $target(handle) [expr $register_value | 0] $AT91C_PIOC_PDR dummy_err

    puts "-I- Init MATRIX to support EBI0 CS1 for SDRAM"
    set register_value [expr [TCL_Read_Int $target(handle) $AT91C_CCFG_EBICSA dummy_err] | $AT91C_MATRIX_CS1A]
    TCL_Write_Int $target(handle) $register_value $AT91C_CCFG_EBICSA dummy_err

    puts "-I- Init SDRAM"
    puts "-I- 1. A minimum pause of 200us is provided to precede any signal toggle"
    set register_value [expr $AT91C_SDRAMC_TXSR_3 | $AT91C_SDRAMC_TRAS_2 | $AT91C_SDRAMC_TRCD_1 | $AT91C_SDRAMC_TRP_1 | $AT91C_SDRAMC_TRC_4 | $AT91C_SDRAMC_TWR_1]
    set register_value [expr $register_value | $AT91C_SDRAMC_NC_9 | $AT91C_SDRAMC_NR_13 | $AT91C_SDRAMC_NB_4_BANKS | $AT91C_SDRAMC_CAS_2 | $AT91C_SDRAMC_DBW_32_BITS]
    TCL_Write_Int $target(handle) $register_value $AT91C_SDRAMC_CR dummy_err

    puts "-I- 2. A Precharge All command is issued to the SDRAM"
    TCL_Write_Int $target(handle) $AT91C_SDRAMC_MODE_PRCGALL_CMD $AT91C_SDRAMC_MR dummy_err
    puts "-I- *pSDRAM = 0;"
    TCL_Write_Int $target(handle) 0 $AT91C_EBI_SDRAM dummy_err

    puts "-I- 3. Eight Auto-refresh are provided"
    TCL_Write_Int $target(handle) $AT91C_SDRAMC_MODE_RFSH_CMD $AT91C_SDRAMC_MR dummy_err
    for {set i 0} {$i < 8} {incr i} {
        puts "-I- *pSDRAM = 0;"
        TCL_Write_Int $target(handle) 0 $AT91C_EBI_SDRAM dummy_err
    }
    
    puts "-I- 4. A mode register cycle is issued to program the SDRAM parameters"
    TCL_Write_Int $target(handle) $AT91C_SDRAMC_MODE_LMR_CMD $AT91C_SDRAMC_MR dummy_err
    puts "-I- *(pSDRAM+0x20) = 0;"
    TCL_Write_Int $target(handle) 0xcafedede [expr $AT91C_EBI_SDRAM + 0x20] dummy_err
    
    puts "-I- 5. Write refresh rate into SDRAMC refresh timer COUNT register"
    TCL_Write_Int $target(handle) [expr $AT91C_SDRAMC_COUNT & 0x65B] $AT91C_SDRAMC_TR dummy_err

    puts "-I- 6. A Normal Mode Command is provided, 3 clocks after tMRD is set"
    TCL_Write_Int $target(handle) $AT91C_SDRAMC_MODE_NORMAL_CMD $AT91C_SDRAMC_MR dummy_err
    puts "-I- *pSDRAM = 0;"
    TCL_Write_Int $target(handle) 0 $AT91C_EBI_SDRAM dummy_err

    set SDRAM_initialized 1
    puts "-I- End of Init_SDRAM_48"
}


################################################################################
#  proc SDRAM::initSDRAM_100
################################################################################
proc SDRAM::initSDRAM_100 {} {

    global AT91C_EBI_SDRAM AT91C_CCFG_EBICSA AT91C_MATRIX_CS1A
    global AT91C_SDRAMC_CR AT91C_SDRAMC_TXSR_8 AT91C_SDRAMC_TRAS_5 AT91C_SDRAMC_TRCD_2 
    global AT91C_SDRAMC_TRC_7 AT91C_SDRAMC_TWR_2 AT91C_SDRAMC_TRP_2
    global AT91C_SDRAMC_CAS_2 AT91C_SDRAMC_DBW_32_BITS AT91C_SDRAMC_NC_9 AT91C_SDRAMC_NR_13 AT91C_SDRAMC_NB_4_BANKS
    global AT91C_SDRAMC_MR AT91C_SDRAMC_MODE_PRCGALL_CMD AT91C_SDRAMC_MODE_RFSH_CMD 
    global AT91C_SDRAMC_MODE_LMR_CMD AT91C_SDRAMC_MODE_NORMAL_CMD
    global AT91C_SDRAMC_TR AT91C_SDRAMC_COUNT
    global AT91C_PIOC_ASR AT91C_PIOC_BSR AT91C_PIOC_PDR

    global target
    set dummy_err 0

    variable SDRAM_initialized
    if {$SDRAM_initialized != 0} {
        puts "-I- SDRAM already initialized"
        return
    }

    puts "-I- Configure PIOC as peripheral (D16/D31)"
    # PIOC16 to 31
    set register_value 0xFFFF0000
    # AT91F_PIO_CfgPeriph
    TCL_Write_Int $target(handle) $register_value $AT91C_PIOC_ASR dummy_err
    TCL_Write_Int $target(handle) 0 $AT91C_PIOC_BSR dummy_err
    # Set in Periph mode
    TCL_Write_Int $target(handle) [expr $register_value | 0] $AT91C_PIOC_PDR dummy_err

    puts "-I- Init MATRIX to support EBI0 CS1 for SDRAM"
    set register_value [expr [TCL_Read_Int $target(handle) $AT91C_CCFG_EBICSA dummy_err] | $AT91C_MATRIX_CS1A]
    TCL_Write_Int $target(handle) $register_value $AT91C_CCFG_EBICSA dummy_err

    puts "-I- Init SDRAM"
    puts "-I- 1. A minimum pause of 200us is provided to precede any signal toggle"
    set register_value [expr $AT91C_SDRAMC_TXSR_8 | $AT91C_SDRAMC_TRAS_5 | $AT91C_SDRAMC_TRCD_2 | $AT91C_SDRAMC_TRP_2 | $AT91C_SDRAMC_TRC_7 | $AT91C_SDRAMC_TWR_2]
    set register_value [expr $register_value | $AT91C_SDRAMC_NC_9 | $AT91C_SDRAMC_NR_13 | $AT91C_SDRAMC_NB_4_BANKS | $AT91C_SDRAMC_CAS_2 | $AT91C_SDRAMC_DBW_32_BITS]
    TCL_Write_Int $target(handle) $register_value $AT91C_SDRAMC_CR dummy_err

    puts "-I- 2. A Precharge All command is issued to the SDRAM"
    TCL_Write_Int $target(handle) $AT91C_SDRAMC_MODE_PRCGALL_CMD $AT91C_SDRAMC_MR dummy_err
    puts "-I- *pSDRAM = 0;"
    TCL_Write_Int $target(handle) 0 $AT91C_EBI_SDRAM dummy_err

    puts "-I- 3. Eight Auto-refresh are provided"
    TCL_Write_Int $target(handle) $AT91C_SDRAMC_MODE_RFSH_CMD $AT91C_SDRAMC_MR dummy_err
    for {set i 0} {$i < 8} {incr i} {
        puts "-I- *pSDRAM = 0;"
        TCL_Write_Int $target(handle) 0 $AT91C_EBI_SDRAM dummy_err
    }

    puts "-I- 4. A mode register cycle is issued to program the SDRAM parameters"
    TCL_Write_Int $target(handle) $AT91C_SDRAMC_MODE_LMR_CMD $AT91C_SDRAMC_MR dummy_err
    puts "-I- *(pSDRAM+0x20) = 0;"
    TCL_Write_Int $target(handle) 0xcafedede [expr $AT91C_EBI_SDRAM + 0x20] dummy_err

    puts "-I- 5. Write refresh rate into SDRAMC refresh timer COUNT register"
    TCL_Write_Int $target(handle) [expr $AT91C_SDRAMC_COUNT & 0x65B] $AT91C_SDRAMC_TR dummy_err

    puts "-I- 6. A Normal Mode Command is provided, 3 clocks after tMRD is set"
    TCL_Write_Int $target(handle) $AT91C_SDRAMC_MODE_NORMAL_CMD $AT91C_SDRAMC_MR dummy_err
    puts "-I- *pSDRAM = 0;"
    TCL_Write_Int $target(handle) 0 $AT91C_EBI_SDRAM dummy_err

    set SDRAM_initialized 1
    puts "-I- End of Init_SDRAM_100"
}


################################################################################
#  proc SDRAM::initSDRAM_133
################################################################################
proc SDRAM::initSDRAM_133 {} {

    global AT91C_EBI_SDRAM AT91C_CCFG_EBICSA AT91C_MATRIX_CS1A
    global AT91C_SDRAMC_CR AT91C_SDRAMC_TXSR_10 AT91C_SDRAMC_TRAS_6 AT91C_SDRAMC_TRCD_3
    global AT91C_SDRAMC_TRC_9 AT91C_SDRAMC_TWR_2 AT91C_SDRAMC_TRP_3
    global AT91C_SDRAMC_CAS_3 AT91C_SDRAMC_DBW_32_BITS AT91C_SDRAMC_NC_9 AT91C_SDRAMC_NR_13 AT91C_SDRAMC_NB_4_BANKS
    global AT91C_SDRAMC_MR AT91C_SDRAMC_MODE_PRCGALL_CMD AT91C_SDRAMC_MODE_RFSH_CMD 
    global AT91C_SDRAMC_MODE_LMR_CMD AT91C_SDRAMC_MODE_NORMAL_CMD
    global AT91C_SDRAMC_TR AT91C_SDRAMC_COUNT
    global AT91C_PIOC_ASR AT91C_PIOC_BSR AT91C_PIOC_PDR

    global target
    set dummy_err 0

    variable SDRAM_initialized
    if {$SDRAM_initialized != 0} {
        puts "-I- SDRAM already initialized"
        return
    }

    puts "-I- Configure PIOC as peripheral (D16/D31)"
    # PIOC16 to 31
    set register_value 0xFFFF0000
    # AT91F_PIO_CfgPeriph
    TCL_Write_Int $target(handle) $register_value $AT91C_PIOC_ASR dummy_err
    TCL_Write_Int $target(handle) 0 $AT91C_PIOC_BSR dummy_err
    # Set in Periph mode
    TCL_Write_Int $target(handle) [expr $register_value | 0] $AT91C_PIOC_PDR dummy_err

    puts "-I- Init MATRIX to support EBI0 CS1 for SDRAM"
    set register_value [expr [TCL_Read_Int $target(handle) $AT91C_CCFG_EBICSA dummy_err] | $AT91C_MATRIX_CS1A]
    TCL_Write_Int $target(handle) $register_value $AT91C_CCFG_EBICSA dummy_err

    puts "-I- Init SDRAM"
    puts "-I- 1. A minimum pause of 200us is provided to precede any signal toggle"
    set register_value [expr $AT91C_SDRAMC_TXSR_10 | $AT91C_SDRAMC_TRAS_6 | $AT91C_SDRAMC_TRCD_3 | $AT91C_SDRAMC_TRP_3 | $AT91C_SDRAMC_TRC_9 | $AT91C_SDRAMC_TWR_2]
    set register_value [expr $register_value | $AT91C_SDRAMC_NC_9 | $AT91C_SDRAMC_NR_13 | $AT91C_SDRAMC_NB_4_BANKS | $AT91C_SDRAMC_CAS_3 | $AT91C_SDRAMC_DBW_32_BITS]
    TCL_Write_Int $target(handle) $register_value $AT91C_SDRAMC_CR dummy_err

    puts "-I- 2. A Precharge All command is issued to the SDRAM"
    TCL_Write_Int $target(handle) $AT91C_SDRAMC_MODE_PRCGALL_CMD $AT91C_SDRAMC_MR dummy_err
    puts "-I- *pSDRAM = 0;"
    TCL_Write_Int $target(handle) 0 $AT91C_EBI_SDRAM dummy_err

    puts "-I- 3. Eight Auto-refresh are provided"
    TCL_Write_Int $target(handle) $AT91C_SDRAMC_MODE_RFSH_CMD $AT91C_SDRAMC_MR dummy_err
    for {set i 0} {$i < 8} {incr i} {
        puts "-I- *pSDRAM = 0;"
        TCL_Write_Int $target(handle) 0 $AT91C_EBI_SDRAM dummy_err
    }

    puts "-I- 4. A mode register cycle is issued to program the SDRAM parameters"
    TCL_Write_Int $target(handle) $AT91C_SDRAMC_MODE_LMR_CMD $AT91C_SDRAMC_MR dummy_err
    puts "-I- *(pSDRAM+0x20) = 0;"
    TCL_Write_Int $target(handle) 0xcafedede [expr $AT91C_EBI_SDRAM + 0x20] dummy_err

    puts "-I- 5. Write refresh rate into SDRAMC refresh timer COUNT register"
    TCL_Write_Int $target(handle) [expr $AT91C_SDRAMC_COUNT & 0x65B] $AT91C_SDRAMC_TR dummy_err

    puts "-I- 6. A Normal Mode Command is provided, 3 clocks after tMRD is set"
    TCL_Write_Int $target(handle) $AT91C_SDRAMC_MODE_NORMAL_CMD $AT91C_SDRAMC_MR dummy_err
    puts "-I- *pSDRAM = 0;"
    TCL_Write_Int $target(handle) 0 $AT91C_EBI_SDRAM dummy_err

    set SDRAM_initialized 1
    puts "-I- End of Init_SDRAM_133"
}


################################################################################
#  proc SDRAM::sendFile
################################################################################
proc SDRAM::sendFile { name addr } {
    
    global valueOfDataForSendFile
    global target
    set dummy_err 0

    variable SDRAM_initialized
    if {$SDRAM_initialized != 1} {
        puts stderr "-E- SDRAM not initialized"
        return
    }

    if {[catch {set f [open $name r]}]} {
        set valueOfDataForSendFile 0
        puts stderr "-E- Can't open file $name"
        return -1
    }
    
    fconfigure $f -translation binary
    
    set size [file size $name]
    puts "-I- File size = $size byte(s)"
    set valueOfDataForSendFile [read $f $size]
    
    close $f

    if {[catch {TCL_Write_Data $target(handle) $addr valueOfDataForSendFile $size dummy_err}]} {
        puts stderr "-E- Can't send data, error in connection"
        set valueOfDataForSendFile 0
        return
    }
    
    set valueOfDataForSendFile 0
}


################################################################################
#  proc SDRAM::receiveFile
################################################################################
proc SDRAM::receiveFile {name addr size} {

    variable SDRAM_initialized
    if {$SDRAM_initialized != 1} {
        puts stderr "-E- SDRAM not initialized"
        return
    }

    global target
    set dummy_err 0

    #read data from target
    if {[catch {set result [TCL_Read_Data $target(handle) $addr $size dummy_err]}]} {
        puts stderr "-E- Can't read data, error in connection"
        return -1
    }

    # put data in a file
    if {[catch {set f2 [open $name w+]}]} {
        puts stderr "-E- Can't open file $name"
        return -1
    }

    fconfigure $f2 -translation binary
    puts -nonewline $f2 $result
    close $f2
}

