#  ----------------------------------------------------------------------------
#          ATMEL Microcontroller Software Support  -  ROUSSET  -
#  ----------------------------------------------------------------------------
#  Copyright (c) 2006, Atmel Corporation
#
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#  this list of conditions and the disclaiimer below.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#  this list of conditions and the disclaimer below in the documentation and/or
#  other materials provided with the distribution. 
#
#  Atmel's name may not be used to endorse or promote products derived from
#  this software without specific prior written permission. 
#
#  DISCLAIMER: THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
#  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
#  DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
#  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
#  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#  ----------------------------------------------------------------------------


################################################################################
################################################################################
## NAMESPACE NANDFLASH
################################################################################
################################################################################
namespace eval NANDFLASH {
    variable StartAddress        [expr 0x0]
    variable MemorySize          [expr 132*1024]
    variable MaxBootSize         [expr 156*1024]
    
    variable array nf
    array set nf {
        comType                      0x3013EC
        monitorArg0                  0x3013F0
        monitorAddr                  0x323000
        monitorArg1                  0x3013F4
        monitorArg2                  0x3013F8
        monitorArg3                  0x3013FC
        monitorArg4                  0x301400
        monitorArg5                  0x301404
        monitorMbx                   0x301400
        bufferAddr                   0x301400
        nandNbBlocks                 0x400
        nandBlockSize                0x20000
        nandSectorSize               0x800
        nandSpareSize                0x40
        nandBusWidth                 0
        monitorName                  "SAM-BA-nand.bin"
    }

    variable array nf_cmd
    array set nf_cmd {
        'NAND_INIT'            0
        'NAND_ERASE'           1
        'NAND_READ'            2
        'NAND_WRITE'           3
        'NAND_ID'              4
        'NAND_WRITE_BLOCK'     5
        'NAND_CHECK_BLOCK'     6
        'NAND_ERASE_FULL'      7
    }

    variable array nf_zone
    array set nf_zone {
        'NF_INFO_ZONE'           0x0
        'NF_BAD_BLOCK'         0xBAD
        'NF_DATA_ZONE'          0xFF
    }
}


################################################################################
#  proc NANDFLASH::Init
################################################################################
proc NANDFLASH::Init { } {
    global AT91C_MATRIX_EBICSA AT91C_MATRIX_EBICSA AT91C_MATRIX_CS3A_SM
    global AT91C_SMC_SETUP3    
    global AT91C_SMC_PULSE3    
    global AT91C_SMC_CYCLE3  
    global AT91C_SMC_CTRL3 AT91C_SMC_READMODE AT91C_SMC_WRITEMODE AT91C_SMC_NWAITM_NWAIT_DISABLE AT91C_SMC_DBW_WIDTH_EIGTH_BITS AT91C_SMC_DBW_WIDTH_SIXTEEN_BITS AT91C_SM_TDF
    global AT91C_PMC_PCER AT91C_ID_PIOC
    global AT91C_PIOC_ODR AT91C_PIO_PC15 AT91C_PIO_PC14 AT91C_PIOC_PER AT91C_PIOC_PPUER
    global AT91C_PIOC_ASR AT91C_PIOC_PDR AT91C_PC0_SMOE AT91C_PC1_SMWE
    global AT91C_PIOC_PER AT91C_PIOC_OER AT91C_PIO_PC14

    global target
    set dummy_err 0

    variable nf
    variable MemorySize
    
    set AT91C_SM_NWE_SETUP      [expr (1 << 0)]
    set AT91C_SM_NCS_WR_SETUP   [expr (0 << 8)]         
    set AT91C_SM_NRD_SETUP      [expr (1 << 16)]
    set AT91C_SM_NCS_RD_SETUP   [expr (0 << 24)]        

    set AT91C_SM_NWE_PULSE      [expr (2 << 0)] 
    set AT91C_SM_NCS_WR_PULSE   [expr (3 << 8)] 
    set AT91C_SM_NRD_PULSE      [expr (2 << 16)]        
    set AT91C_SM_NCS_RD_PULSE   [expr (4 << 24)]        

    set AT91C_SM_NWE_CYCLE      [expr (3 << 0)] 
    set AT91C_SM_NRD_CYCLE      [expr (5 << 16)]        

    set AT91C_SM_TDF            [expr (1 << 16) ]       
    
    puts "-I- Enable the address range of CS3 in HMATRIX user interface"
    TCL_Write_Int $target(handle) [expr [TCL_Read_Int $target(handle) $AT91C_MATRIX_EBICSA dummy_err] | $AT91C_MATRIX_CS3A_SM] $AT91C_MATRIX_EBICSA dummy_err

    puts "-I- Configure SMC CS3"
    TCL_Write_Int $target(handle) [expr $AT91C_SM_NWE_SETUP | $AT91C_SM_NCS_WR_SETUP | $AT91C_SM_NRD_SETUP | $AT91C_SM_NCS_RD_SETUP] $AT91C_SMC_SETUP3 dummy_err
    
    TCL_Write_Int $target(handle) [expr $AT91C_SM_NWE_PULSE | $AT91C_SM_NCS_WR_PULSE | $AT91C_SM_NRD_PULSE | $AT91C_SM_NCS_RD_PULSE] $AT91C_SMC_PULSE3 dummy_err
    
    TCL_Write_Int $target(handle) [expr $AT91C_SM_NWE_CYCLE | $AT91C_SM_NRD_CYCLE] $AT91C_SMC_CYCLE3 dummy_err
    
    TCL_Write_Int $target(handle) [expr $AT91C_SMC_READMODE | $AT91C_SMC_WRITEMODE | $AT91C_SMC_NWAITM_NWAIT_DISABLE | \
                                       $AT91C_SMC_DBW_WIDTH_EIGTH_BITS | $AT91C_SM_TDF] $AT91C_SMC_CTRL3 dummy_err
    
    puts "-I- Clock PIOC"
    TCL_Write_Int $target(handle) [expr (1 << $AT91C_ID_PIOC)] $AT91C_PMC_PCER dummy_err

    puts "-I- Configure Ready/Busy signal"
    TCL_Write_Int $target(handle) $AT91C_PIO_PC15 $AT91C_PIOC_ODR dummy_err
    TCL_Write_Int $target(handle) $AT91C_PIO_PC15 $AT91C_PIOC_PER dummy_err
    puts "-I- Configure pull-up"
    TCL_Write_Int $target(handle) $AT91C_PIO_PC15 $AT91C_PIOC_PPUER dummy_err
    
    puts "-I-Enable PC0=SMOE line PC1=SMWE, A21=CLE, A22=ALE"
    TCL_Write_Int $target(handle) [expr $AT91C_PC0_SMOE | $AT91C_PC1_SMWE] $AT91C_PIOC_ASR dummy_err
    TCL_Write_Int $target(handle) [expr $AT91C_PC0_SMOE | $AT91C_PC1_SMWE] $AT91C_PIOC_PDR dummy_err
    
    puts "-I- Enable NandFlash"
    TCL_Write_Int $target(handle) $AT91C_PIO_PC14 $AT91C_PIOC_PER dummy_err
    TCL_Write_Int $target(handle) $AT91C_PIO_PC14 $AT91C_PIOC_OER dummy_err
    
    puts "-I- Initialize SDRAM"
    SDRAM::initSDRAM_48

    puts "-I- ReadId"
    NANDFLASH::SendMonitor

    set fullID [NANDFLASH::ReadId]
    set Nand_ID [format "%08x" $fullID]
    puts "-I- NandFlash ID : 0x$Nand_ID"
    set manID [string range $Nand_ID 4 5]
    set devID [string range $Nand_ID 6 7]
    set extID [expr $fullID / 16777216]

    switch $manID {
        "2c" {set manName "Micron"}
        "ec" {set manName "Samsung"}
        "98" {set manName "Toshiba"}
        "04" {set manName "Fujitsu"}
        "8f" {set manName "National"}
        "20" {set manName "STMicro"}
        default {
            puts stderr "-E- Manufacturer not supported..."
            return
        }
    }

    set nand_small_blocks 0
    
    if {$devID == "76"} {
            set nf(nandNbBlocks)          0x1000
            set nf(nandBlockSize)         0x4000
            set nf(nandSectorSize)         0x200
            set nf(nandSpareSize)           0x10
            set nf(nandBusWidth)               0
            set nf(monitorName)                "SAM-BA-nand-small.bin"
            set nand_small_blocks 1
    }
    
    if {$nand_small_blocks == 0} {
        switch $devID {
            "a1" { set nf(nandNbBlocks) 0x400}
            "f1" { set nf(nandNbBlocks) 0x400}
            "b1" { set nf(nandNbBlocks) 0x400}
            "c1" { set nf(nandNbBlocks) 0x400}

            "aa" { set nf(nandNbBlocks) 0x800}
            "da" { set nf(nandNbBlocks) 0x800}
            "ba" { set nf(nandNbBlocks) 0x800}
            "ca" { set nf(nandNbBlocks) 0x800}

            "ac" { set nf(nandNbBlocks) 0x1000}
            "dc" { set nf(nandNbBlocks) 0x1000}
            "bc" { set nf(nandNbBlocks) 0x1000}
            "cc" { set nf(nandNbBlocks) 0x1000}

            "a3" { set nf(nandNbBlocks) 0x2000}
            "d3" { set nf(nandNbBlocks) 0x2000}
            "b3" { set nf(nandNbBlocks) 0x2000}
            "c3" { set nf(nandNbBlocks) 0x2000}

            "a5" { set nf(nandNbBlocks) 0x4000}
            "d5" { set nf(nandNbBlocks) 0x4000}
            "b5" { set nf(nandNbBlocks) 0x4000}
            "c5" { set nf(nandNbBlocks) 0x4000}

            default {
                puts stderr "-E- NandFlash not supported..."
                return}
        }

        set nf(nandSectorSize) [format "0x%x" [expr 0x400 << [expr $extID & 0x1]]]
        puts "-I- Sector size : $nf(nandSectorSize)"

        set nf(nandBlockSize) [format "0x%x" [expr 0x10000 << [expr [expr $extID / 16] & 0x3]]]
        puts "-I- Block size : $nf(nandBlockSize)"

        set nf(nandSpareSize) 0x40
        puts "-I- Spare size : $nf(nandSpareSize)"

        set nf(nandBusWidth) [format "0x%x" [expr [expr $extID / 64] & 0x1]]
        puts "-I- Bus width size : $nf(nandBusWidth)"

        set nf(monitorName) "SAM-BA-nand.bin"
    }
    
    set MemorySize [expr $nf(nandNbBlocks) * $nf(nandBlockSize)]

    set buswidth [expr ($nf(nandBusWidth) + 1) * 8]
    set memSizeMB [expr $MemorySize / 1024 / 1024]
    puts "-I- NandFlash $manName $buswidth bits $memSizeMB MBytes"
    
    if {$nf(nandBusWidth) == 1} {
        TCL_Write_Int $target(handle) [expr $AT91C_SMC_READMODE | $AT91C_SMC_WRITEMODE | $AT91C_SMC_NWAITM_NWAIT_DISABLE | \
                                           $AT91C_SMC_DBW_WIDTH_SIXTEEN_BITS | $AT91C_SM_TDF] $AT91C_SMC_CTRL3 dummy_err
    }

    NANDFLASH::SendMonitor
}

################################################################################
#  proc NANDFLASH::SendBootFileGUI
################################################################################
proc NANDFLASH::sendBootFileGUI {} {
    global softwareStatusLabelVariable
    global backupDirPath
    variable nf
    variable MaxBootSize
    global commandLineMode
    global conWindows
    
    set fileName [tk_getOpenFile -parent $conWindows -initialdir  $backupDirPath -defaultextension ".bin" -filetypes {{"Bin Files" {.bin}} {"All Files" {*.*}}}]
    if {$fileName == ""} {
        puts stderr "-E- No File Selected"
        return
    }
    
    if {[catch {set size [file size $fileName]}]} {
        puts stderr "-E- Can't open file $fileName"
        return
    }
    
    if {$size > $MaxBootSize} {
        puts stderr "-E- Unauthorized Boot File Size"
        return
    }
    
    puts "-I- File size = $size byte(s)"
    
    file copy -force $fileName "tmpSendBootFile.bin"
    
    if { [catch {set f2 [open "tmpSendBootFile.bin" RDWR ]}] } {
        puts stderr "-E- File Error"
        return
    }
    
    # Wait window for loading
    set softwareStatusLabelVariable "Sending Boot File to the target ..."
    if {$commandLineMode == 0} {
        waitWindows::createWindow "Sending Boot File to the target" "hourglass.gif"
        tkwait visibility .topWaitWindow
    }
    # Modify 6th vector with file size
    global target
    set dummy_err 0

    fconfigure $f2 -translation binary
    seek $f2 20 start
    TCL_Write_Int $target(handle) [expr $size] $nf(bufferAddr) dummy_err
    puts -nonewline $f2 [TCL_Read_Data $target(handle) $nf(bufferAddr) 4 dummy_err]
    seek $f2 0 start
    
    # Write Modified File
    NANDFLASH::sendFile "tmpSendBootFile.bin" 0
    
    close $f2
    file delete "tmpSendBootFile.bin"

    if {$commandLineMode == 0} {
        waitWindows::destroyWindow   
    }
    set softwareStatusLabelVariable ""

    set backupDirPath [file dirname $fileName]
    puts "-I- Boot File Written Successfully"
}


################################################################################
#  proc NANDFLASH::SendBootFile
################################################################################
proc NANDFLASH::sendBootFile {fileNameWrite} {
    variable nf
    variable MaxBootSize
    
    
    if {$fileNameWrite == ""} {
        puts stderr "-E- No File Selected"
        return
    }
    
    if {[catch {set size [file size $fileNameWrite]}]} {
        puts stderr "-E- Can't open file $fileNameWrite"
        return
    }
    
    if {$size > $MaxBootSize} {
        puts stderr "-E- Unauthorized Boot File Size"
        return
    }
    
    puts "-I- File size = $size byte(s)"
    
    file copy -force $fileNameWrite "tmpSendBootFile.bin"
    
    if { [catch {set f2 [open "tmpSendBootFile.bin" RDWR ]}] } {
        puts stderr "-E- File Error"
        return
    }
    
    # Modify 6th vector with file size
    global target
    set dummy_err 0

    fconfigure $f2 -translation binary
    seek $f2 20 start
    TCL_Write_Int $target(handle) [expr $size] $nf(bufferAddr) dummy_err
    puts -nonewline $f2 [TCL_Read_Data $target(handle) $nf(bufferAddr) 4 dummy_err]
    seek $f2 0 start
    
    # Write Modified File
    NANDFLASH::sendFile "tmpSendBootFile.bin" 0
    
    close $f2
    file delete "tmpSendBootFile.bin"

    set backupDirPath [file dirname $fileNameWrite]
    puts "-I- Boot File Written Successfully"
}


################################################################################
#  proc NANDFLASH::sendFile
################################################################################
proc NANDFLASH::sendFile { name address } {
    variable nf
    variable nf_zone

    global target
    set dummy_err 0
    
    puts "-I- NANDFLASH::sendFile"
    
    # Sanity check
    if {[expr $address % $nf(nandBlockSize)]} {
        puts stderr [format "-E- The address must be block boundary aligned (0x%08x)" $nf(nandBlockSize)]
        return
    }
    
    # Open the file 
    if { [catch {set fHandle [open $name r]}] } {

        puts stderr "-E- Can't open file $name"
        return
    }
    
    fconfigure $fHandle -translation binary
    
    set length [file size $name]
    puts "-I- File size = $length byte(s)"
    
    # Initialize the block offset
    set blockIdx [expr $address / $nf(nandBlockSize)]
    set startBlock $blockIdx
    # Initialize the number of bad blocks
    set badBlock 0
    set blockRead 0

    while {$length > 0} {
        # Read a buffer corresponding to a block in the origin file
        if {$length < $nf(nandBlockSize)} {
            set sizeToRead $length
        } else {
            set sizeToRead $nf(nandBlockSize)
        }
        
        # Adjust the number of sectors to program
        set nbSector [expr $sizeToRead / $nf(nandSectorSize)]
        if {[expr $sizeToRead % $nf(nandSectorSize)]} {
            incr nbSector
        }
        
        #Loop until a valid block has been written
        set newBlock 1
        while {$newBlock == 1} {
            # Reset the error flag
            set blockError 0
            
        # Check if this block is tagged as bad
            if {[catch {NANDFLASH::CheckBlock $blockIdx}]} {
                set blockAddr [format "0x%08X" [expr $blockIdx * $nf(nandBlockSize)]]
                puts "-I- Block $blockIdx is bad ($blockAddr) ... skip block"
                incr badBlock
                set blockError 1
            }
            
            # Erase the block
            if {$blockError == 0} {
                if {[catch {NANDFLASH::EraseBlock $blockIdx}]} {
                    set blockError 1
                } else {
                
                    # Position the pointer in the file
                    seek $fHandle [expr $blockRead * $nf(nandBlockSize)] start
                    # Read the sector
                    set buffer [read $fHandle $sizeToRead]
                    
                    if {[catch {NANDFLASH::WriteBlock $blockIdx buffer $sizeToRead}]} {
                            set blockError 1
                    }
                    
                    if {$blockError == 0} {
                        # If the block is valid exit
                        set newBlock 0
                        
                    } else {
                        set blockAddr [format "0x%08X" [expr $blockIdx * $nf(nandBlockSize)]]
                        puts "-I- Block $blockIdx ($blockAddr) found bad after write operation ... tag block as bad"
                        
                        #Mark the block as invalid
                        NANDFLASH::WriteSector $blockIdx 0 buffer 0 $nf_zone('NF_INFO_ZONE')
                        #increment the number of bad blocks
                        incr badBlock
                    }
                }
            }
            incr blockIdx
        }
        
        # Decrement length
        set length [expr $length - $sizeToRead]
        incr blockRead
    }

    set blockIdx [expr $blockIdx - 1]

    puts "-I- Blocks $startBlock to $blockIdx are programmed"
    puts "-I- Number of bad blocks: $badBlock"
    
    close $fHandle
}

################################################################################
#  proc NANDFLASH::receiveFile
################################################################################
proc NANDFLASH::receiveFile {name address size} {
    variable nf
    variable nf_zone

    global target
    set dummy_err 0
   
    # put data in a file
    if { [catch {set fHandle [open $name w+]}] } {
        puts stderr "-E- Can't open file $name"
        return -1
    }
    fconfigure $fHandle -translation binary
    
    # Initialize the block offset
    set blockIdx [expr $address / $nf(nandBlockSize)]
    # Initialize the number of bad blocks
    set badBlock 0
    set blockRead 0
    
    set length $size
    
    while {$length > 0} {
        # Read a buffer corresponding to a block in the origin file
        if {$length < $nf(nandBlockSize)} {
            set sizeToRead $length
        } else {
            set sizeToRead $nf(nandBlockSize)
        }

        # Adjust the number of sectors to program
        set nbSector [expr $sizeToRead / $nf(nandSectorSize)]
        if {[expr $sizeToRead % $nf(nandSectorSize)]} {
            incr nbSector
        }
        
        set newBlock 1
        #Loop until a valid block has been written
        while {$newBlock == 1} {
            # Reset the error flag
            set blockError 0
            
            # Check if this is an invalid block by reading the spare bytes
            # of the two first sectors
            catch {NANDFLASH::ReadSector $blockIdx 0 $nf_zone('NF_INFO_ZONE')} page0Err
            if {($page0Err eq $nf_zone('NF_BAD_BLOCK'))} {
                set blockError 1
            }
            
            # Read the sectors
            for {set sectorIdx 0} {($sectorIdx < $nbSector) && ($blockError == 0)} {incr sectorIdx} {
                set sectorSize [expr $sizeToRead - ($sectorIdx * $nf(nandSectorSize))]
                if {$sectorSize < $nf(nandSectorSize)} {
                    set sectorSize  [expr $sizeToRead - ($sectorIdx * $nf(nandSectorSize))]
                } else  {
                    set sectorSize $nf(nandSectorSize)
                }

                # Read the sector
                catch {NANDFLASH::ReadSector $blockIdx $sectorIdx $nf_zone('NF_DATA_ZONE')}

                set buffer [TCL_Read_Data $target(handle) $nf(monitorMbx) $sectorSize dummy_err]
                puts -nonewline $fHandle $buffer
            }
            
            if {$blockError == 0} {
                # If the block is valid exit
                set newBlock 0
            }
            incr blockIdx
        }
        
        # Decrement length
        set length [expr $length - $sizeToRead]
        incr blockRead
    }
    
    close $fHandle
}

################################################################################
#  proc NANDFLASH::SendMonitor
################################################################################
proc NANDFLASH::SendMonitor { } {
    variable nf
    variable nf_cmd

    global target
    global libPath
    set dummy_err 0

    set fileNameRead "$libPath(extLib)/$target(board)/$nf(monitorName)"
    # Send the monitor
    if { [catch {set f [open $fileNameRead r]}] } {
        puts stderr "-E- Can't open file $nf(monitorName)"
        error "-E- Can't open file $nf(monitorName)"
        return 
    }
    
    fconfigure $f -translation binary
    set size [file size $fileNameRead]
    set valueOfDataForSendFile [read $f $size]
    TCL_Write_Data $target(handle) $nf(monitorAddr) valueOfDataForSendFile $size dummy_err
    close $f

    TCL_Write_Int $target(handle) $target(comType) $nf(comType) dummy_err
    TCL_Write_Int $target(handle) $nf_cmd('NAND_INIT') $nf(monitorArg0) dummy_err
    TCL_Write_Int $target(handle) $nf(nandNbBlocks) $nf(monitorArg1) dummy_err
    TCL_Write_Int $target(handle) $nf(nandBlockSize) $nf(monitorArg2) dummy_err
    TCL_Write_Int $target(handle) $nf(nandSectorSize) $nf(monitorArg3) dummy_err
    TCL_Write_Int $target(handle) $nf(nandSpareSize) $nf(monitorArg4) dummy_err
    TCL_Write_Int $target(handle) $nf(nandBusWidth) $nf(monitorArg5) dummy_err

    catch {TCL_Go $target(handle) $nf(monitorAddr) dummy_err}
}

################################################################################
#  proc NANDFLASH::ReadId
################################################################################
proc NANDFLASH::ReadId {  } {
    variable nf
    variable nf_cmd

    global target
    set dummy_err 0

    # Set the command arguments    
    TCL_Write_Int $target(handle) $target(comType) $nf(comType) dummy_err    
    TCL_Write_Int $target(handle) $nf_cmd('NAND_ID') $nf(monitorArg0) dummy_err
    
    # Run the command
    catch {TCL_Go $target(handle) $nf(monitorAddr) dummy_err}

    # Test the ID
    return [TCL_Read_Int $target(handle) $nf(monitorArg0) dummy_err]
}


################################################################################
#  proc NANDFLASH::ReadSector
################################################################################
proc NANDFLASH::ReadSector { blockIdx SectorIdx spare } {
    variable nf
    variable nf_cmd
    variable nf_zone

    global target
    set dummy_err 0

    # Set the command arguments    
    TCL_Write_Int $target(handle) $target(comType) $nf(comType) dummy_err
    TCL_Write_Int $target(handle) $nf_cmd('NAND_READ') $nf(monitorArg0) dummy_err
    TCL_Write_Int $target(handle) $blockIdx $nf(monitorArg1) dummy_err
    TCL_Write_Int $target(handle) $SectorIdx $nf(monitorArg2) dummy_err
    if {$spare == $nf_zone('NF_DATA_ZONE')} {
        TCL_Write_Int $target(handle) [expr 0xFF] $nf(monitorArg3) dummy_err
    } else {
        TCL_Write_Int $target(handle) [expr 0x00] $nf(monitorArg3) dummy_err
    }

    # Run the command
    catch {TCL_Go $target(handle) $nf(monitorAddr) dummy_err}

    # Test the result
    if {[TCL_Read_Int $target(handle) $nf(monitorArg0) dummy_err] == [expr 0xBAD]} {
        error $nf_zone('NF_BAD_BLOCK')
    }

    # Return the sector data
    return [TCL_Read_Data $target(handle) $nf(monitorMbx) $nf(nandSectorSize) dummy_err]
}

################################################################################
#  proc NANDFLASH::CheckBlock
################################################################################
proc NANDFLASH::CheckBlock { blockIdx } {
    variable nf
    variable nf_cmd
    variable nf_zone

    global target
    set dummy_err 0
   
    # Set the command arguments    
    TCL_Write_Int $target(handle) $target(comType) $nf(comType) dummy_err    
    TCL_Write_Int $target(handle) $nf_cmd('NAND_CHECK_BLOCK') $nf(monitorArg0) dummy_err
    TCL_Write_Int $target(handle) $blockIdx $nf(monitorArg1) dummy_err

    # Run the command
    catch {TCL_Go $target(handle) $nf(monitorAddr) dummy_err}

    # Test the result
    if {[TCL_Read_Int $target(handle) $nf(monitorArg0) dummy_err] == [expr 0xBAD]} {
        error $nf_zone('NF_BAD_BLOCK')
    }
}

################################################################################
#  proc NANDFLASH::EraseBlock
################################################################################
proc NANDFLASH::EraseBlock { blockIdx } {
    variable nf
    variable nf_cmd
    variable nf_zone

    global target
    set dummy_err 0
   
    # Set the command arguments    
    TCL_Write_Int $target(handle) $target(comType) $nf(comType) dummy_err
    TCL_Write_Int $target(handle) $nf_cmd('NAND_ERASE') $nf(monitorArg0) dummy_err
    TCL_Write_Int $target(handle) $blockIdx $nf(monitorArg1) dummy_err
    
    # Run the command
    catch {TCL_Go $target(handle) $nf(monitorAddr) dummy_err}
        
    # Test the result
    if {[TCL_Read_Int $target(handle) $nf(monitorArg0) dummy_err] == [expr 0xBAD]} {
        error $nf_zone('NF_BAD_BLOCK')
    }
}

################################################################################
#  proc NANDFLASH::EraseBlockFull
################################################################################
proc NANDFLASH::EraseBlockFull { blockIdx } {
    variable nf
    variable nf_cmd
    variable nf_zone

    global target
    set dummy_err 0
   
    # Set the command arguments    
    TCL_Write_Int $target(handle) $target(comType) $nf(comType) dummy_err
    TCL_Write_Int $target(handle) $nf_cmd('NAND_ERASE_FULL') $nf(monitorArg0) dummy_err
    TCL_Write_Int $target(handle) $blockIdx $nf(monitorArg1) dummy_err
    
    # Run the command
    catch {TCL_Go $target(handle) $nf(monitorAddr) dummy_err}
        
    # Test the result
    if {[TCL_Read_Int $target(handle) $nf(monitorArg0) dummy_err] == [expr 0xBAD]} {
        error $nf_zone('NF_BAD_BLOCK')
    }
}

################################################################################
#  proc NANDFLASH::WriteBlock
################################################################################
proc NANDFLASH::WriteBlock { blockIdx bufferName blockSize } {
    variable nf
    variable nf_cmd
    variable nf_zone
    upvar $bufferName buffer
    
    global target
    set dummy_err 0

    # Set the command arguments
    TCL_Write_Int $target(handle) $target(comType) $nf(comType) dummy_err
    TCL_Write_Int $target(handle) $nf_cmd('NAND_WRITE_BLOCK') $nf(monitorArg0) dummy_err
    TCL_Write_Int $target(handle) $blockIdx $nf(monitorArg1) dummy_err
    
    # Prepare the buffer
    TCL_Write_Data $target(handle) $nf(monitorMbx) buffer $blockSize dummy_err
    
    # Run the command
    catch {TCL_Go $target(handle) $nf(monitorAddr) dummy_err}

    # Test the result
    if {[TCL_Read_Int $target(handle) $nf(monitorArg0) dummy_err] == [expr 0xBAD]} {
        error $nf_zone('NF_BAD_BLOCK')
    }
}

################################################################################
#  proc NANDFLASH::WriteSector
################################################################################
proc NANDFLASH::WriteSector { blockIdx SectorIdx bufferName sectorSize spare } {
    variable nf
    variable nf_cmd
    variable nf_zone
    upvar $bufferName buffer
    
    global target
    set dummy_err 0

    # Set the command arguments    
    TCL_Write_Int $target(handle) $target(comType) $nf(comType) dummy_err
    TCL_Write_Int $target(handle) $nf_cmd('NAND_WRITE') $nf(monitorArg0) dummy_err
    TCL_Write_Int $target(handle) $blockIdx $nf(monitorArg1) dummy_err
    TCL_Write_Int $target(handle) $SectorIdx $nf(monitorArg2) dummy_err
    if {$spare == $nf_zone('NF_DATA_ZONE')} {
        TCL_Write_Int $target(handle) [expr 0xFF] $nf(monitorArg3) dummy_err
    } else {
        TCL_Write_Int $target(handle) [expr 0x00] $nf(monitorArg3) dummy_err
    }
    
    #prepare the buffer
    TCL_Write_Data $target(handle) $nf(monitorMbx) buffer $sectorSize dummy_err
    
    # Run the command
    catch {TCL_Go $target(handle) $nf(monitorAddr) dummy_err}

    # Test the result
    if {[TCL_Read_Int $target(handle) $nf(monitorArg0) dummy_err] == [expr 0xBAD]} {
        error $nf_zone('NF_BAD_BLOCK')
    }
}

################################################################################
#  proc NANDFLASH::EraseAllNandFlash
################################################################################
proc NANDFLASH::EraseAllNandFlash { } {
    variable nf
    global commandLineMode

    # Wait window for erasing
    if {$commandLineMode == 0} {
        waitWindows::createWindow "Erasing whole NandFlash" "erase.gif"
        tkwait visibility .topWaitWindow
    }

    global softwareStatusLabelVariable
    set softwareStatusLabelVariable "Erasing whole NandFlash ..."

    puts "-I- Erasing whole NandFlash ... Please Wait..."

    # Initialize the number of bad blocks
    set badBlock 0
    set blockIdx 0
    while { $blockIdx < $nf(nandNbBlocks) } {
    # Erase the block
        if {[catch {NANDFLASH::EraseBlock $blockIdx}]} {
            puts "-I- Block $blockIdx NOT erased"
        incr badBlock
        }
        incr blockIdx
    }

    if {$commandLineMode == 0} {
        waitWindows::destroyWindow
    }
    set softwareStatusLabelVariable ""
 
    puts "-I- NandFlash erased"
    puts "-I- $badBlock bad block(s) detected"
}

################################################################################
#  proc NANDFLASH::EraseAllNandFlashFull
################################################################################
proc NANDFLASH::EraseAllNandFlashFull { } {
    variable nf
    global commandLineMode

    # Wait window for erasing
    if {$commandLineMode == 0} {
        waitWindows::createWindow "Erasing whole NandFlash" "erase.gif"
        tkwait visibility .topWaitWindow
    }

    global softwareStatusLabelVariable
    set softwareStatusLabelVariable "Erasing whole NandFlash ..."

    puts "-I- Erasing whole NandFlash ... Please Wait..."

    # Initialize the number of bad blocks
    set badBlock 0
    set blockIdx 0
    while { $blockIdx < $nf(nandNbBlocks) } {
    # Erase the block
        if {[catch {NANDFLASH::EraseBlockFull $blockIdx}]} {
        puts "-I- Block $blockIdx NOT erased"
        incr badBlock
        }
        incr blockIdx
    }

    if {$commandLineMode == 0} {
        waitWindows::destroyWindow
    }
    set softwareStatusLabelVariable ""
 
    puts "-I- NandFlash erased"
    puts "-I- $badBlock bad block(s) detected"
}

################################################################################
#  proc NANDFLASH::BadBlockList
################################################################################
proc NANDFLASH::BadBlockList { } {
    variable nf
    variable nf_cmd
    variable nf_zone
    global target
    set dummy_err 0
    global commandLineMode
    set buffer 0
    set blockIdx 0
    set badBlock 0
    set blockAddr 0

    # Wait window for operation
    if {$commandLineMode == 0} {
        waitWindows::createWindow "Check bad blocks on NandFlash ..." "hourglass.gif"
        tkwait visibility .topWaitWindow
    }

    global softwareStatusLabelVariable
    set softwareStatusLabelVariable "Check bad blocks on NandFlash ..."

    puts "-I- Check bad blocks on NandFlash ... Please Wait..."

    while { $blockIdx < $nf(nandNbBlocks) } {
        if {[catch {NANDFLASH::CheckBlock $blockIdx}]} {
        set blockAddr [format "0x%08X" [expr $blockIdx * $nf(nandBlockSize)]]
        puts "-I- Block $blockIdx is bad ($blockAddr)"
        incr badBlock
    }
        incr blockIdx
    }
    
    puts "-I- $badBlock bad block(s) detected"

    if {$commandLineMode == 0} {
        waitWindows::destroyWindow
    }
    set softwareStatusLabelVariable ""
}

################################################################################
#  proc NANDFLASH::TagBadBlock
################################################################################
proc NANDFLASH::TagBadBlock { blockIdx } {
    variable nf
    variable nf_zone

    set buffer "12345678912345678912345678912346464644"

    if {$blockIdx > $nf(nandNbBlocks)} {
        puts "-E- Block Id ($blockIdx) exceeds NandFlash max block number ($nf(nandNbBlocks))"
        return
    }

    #Mark the block as invalid
    NANDFLASH::WriteSector $blockIdx 0 buffer 0 $nf_zone('NF_INFO_ZONE')

    puts "-I- Block $blockIdx tagged as bad"
}
