#  ----------------------------------------------------------------------------
#          ATMEL Microcontroller Software Support  -  ROUSSET  -
#  ----------------------------------------------------------------------------
#  Copyright (c) 2006, Atmel Corporation
#
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  - Redistributions of source code must retain the above copyright notice,
#  this list of conditions and the disclaiimer below.
#
#  - Redistributions in binary form must reproduce the above copyright notice,
#  this list of conditions and the disclaimer below in the documentation and/or
#  other materials provided with the distribution. 
#
#  Atmel's name may not be used to endorse or promote products derived from
#  this software without specific prior written permission. 
#
#  DISCLAIMER: THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
#  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
#  DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
#  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
#  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#  ----------------------------------------------------------------------------


if { [ catch { source "$libPath(extLib)/$target(board)/SRAM.tcl"} errMsg] } {
    messageDialg  error.gif "Memory Description File not found:\n$errMsg" "File not found" ok
    exit
}

if { [ catch { source "$libPath(extLib)/$target(board)/SDRAM.tcl"} errMsg] } {
    messageDialg  error.gif "Memory Description File not found:\n$errMsg" "File not found" ok
    exit
}

if { [ catch { source "$libPath(extLib)/$target(board)/FLASH.tcl"} errMsg] } {
    messageDialg  error.gif "Memory Description File not found:\n$errMsg" "File not found" ok
    exit
}

if { [ catch { source "$libPath(extLib)/$target(board)/NANDFLASH.tcl"} errMsg] } {
    messageDialg  error.gif "Memory Description File not found:\n$errMsg" "File not found" ok
    exit
}

if { [ catch { source "$libPath(extLib)/$target(board)/DATAFLASH.tcl"} errMsg] } {
    messageDialg  error.gif "Memory Description File not found:\n$errMsg" "File not found" ok
    exit
}


array set memoryAlgo {
    "SRAM"                    "::at91sam9xe512_sram"
    "Flash"                   "::at91sam9xe512_flash"
    "SDRAM"                   "::at91sam9xe512_sdram"
    "DataFlash AT45DB/DCB"    "::at91sam9xe512_dataflash"
    "NandFlash"               "::at91sam9xe512_nandflash"
    "Peripheral"              "::at91sam9xe512_peripheral"
    "ROM"                     "::at91sam9xe512_rom"
    "REMAP"                   "::at91sam9xe512_remap"
}

array set at91sam9xe512_sram {
    dftDisplay  1
    dftDefault  0
    dftAddress  0x300000
    dftSize     0x8000
    dftSend     "SRAM::sendFile"
    dftReceive  "SRAM::receiveFile"
    dftScripts  ""
}

array set at91sam9xe512_flash {
    dftDisplay  1
    dftDefault  1
    dftAddress  0x200000
    dftSize     0x80000
    dftSend     "FLASH::sendFile"
    dftReceive  "FLASH::receiveFile"
    dftScripts  "::at91sam9xe512_flash_scripts"
}

array set at91sam9xe512_flash_scripts {
    "Erase All Flash"                      "FLASH::ScriptEraseAllFlash"
    "Enable Security Bit"                  "FLASH::ScriptSetSecurityBit"
    "Disable Security Bit"                 "FLASH::ScriptClearSecurityBit"
    "Enable BrownOut Detector (GPNVM1)"    "FLASH::ScriptGPNMV 0"
    "Disable BrownOut Detector (GPNVM1)"   "FLASH::ScriptGPNMV 1"
    "Enable BrownOut Reset (GPNVM2)"       "FLASH::ScriptGPNMV 2"
    "Disable BrownOut Reset (GPNVM2)"      "FLASH::ScriptGPNMV 3"
    "Boot from Flash (GPNVM2)"             "FLASH::ScriptGPNMV 4"
    "Boot from ROM (GPNVM2)"               "FLASH::ScriptGPNMV 5"
    "Get Lock Bits"                        "FLASH::ScriptGetLockBit"
    "Get Fuse Bits"                        "FLASH::ScriptGetFuseBit"
}

array set at91sam9xe512_sdram {
    dftDisplay  1
    dftDefault  0
    dftAddress  0x20000000
    dftSize     0x4000000
    dftSend     "SDRAM::sendFile"
    dftReceive  "SDRAM::receiveFile"
    dftScripts  "::at91sam9xe512_sdram_scripts"
}

array set at91sam9xe512_sdram_scripts {
    "Enable SDRAM 48 Hz (default)"   "SDRAM::initSDRAM_48"
    "Enable SDRAM 100 MHz"           "SDRAM::initSDRAM_100"
    "Enable SDRAM 133 MHz"           "SDRAM::initSDRAM_133"
}


array set at91sam9xe512_nandflash {
    dftDisplay  1
    dftDefault  0
    dftAddress  0x0
    dftSize     "$NANDFLASH::MemorySize"
    dftSend     "NANDFLASH::sendFile"
    dftReceive  "NANDFLASH::receiveFile"
    dftScripts  "::at91sam9xe512_nandflash_scripts"
}

array set at91sam9xe512_nandflash_scripts {
    "NandFlash Init"                "NANDFLASH::Init"
    "NandFlash Erase All"           "NANDFLASH::EraseAllNandFlash"
    "NandFlash List Bad Blocks"     "NANDFLASH::BadBlockList"
}

array set at91sam9xe512_dataflash {
    dftDisplay  1
    dftDefault  0
    dftAddress  0x0
    dftSize     "$DATAFLASH::MemorySize"
    dftSend     "DATAFLASH::sendFile"
    dftReceive  "DATAFLASH::receiveFile"
    dftScripts  "::at91sam9xe512_dataflash_scripts"
}

array set at91sam9xe512_dataflash_scripts {
    "Enable Dataflash on CS0"   "DATAFLASH::SelectDataflash AT91C_SPI0_CS0"
    "Enable Dataflash on CS1"   "DATAFLASH::SelectDataflash AT91C_SPI0_CS1"
    "Erase All"                 "DATAFLASH::EraseAllDataFlash"
}

array set at91sam9xe512_peripheral {
    dftDisplay  0
    dftDefault  0
    dftAddress  0xF0000000
    dftSize     0x10000000
    dftSend     ""
    dftReceive  ""
    dftScripts  ""
}

array set at91sam9xe512_rom {
    dftDisplay  0
    dftDefault  0
    dftAddress  0x000000
    dftSize     0x8000
    dftSend     ""
    dftReceive  ""
    dftScripts  ""
}

array set at91sam9xe512_remap {
    dftDisplay  0
    dftDefault  0
    dftAddress  0x00000000
    dftSize     0x8000
    dftSend     ""
    dftReceive  ""
    dftScripts  ""
}
