//@+leo-ver=4
//@+node:@file dataflash.c
//@<<DISCLAIMER>>
//@+node:<<DISCLAIMER>>
//  ----------------------------------------------------------------------------
//          ATMEL Microcontroller Software Support  -  ROUSSET  -
//  ----------------------------------------------------------------------------
//  DISCLAIMER:  THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
//  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
//  DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
//  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
//  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
//  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
//  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//  ----------------------------------------------------------------------------
//@-node:<<DISCLAIMER>>
//@nl

#include "main.h"
#include "dataflash.h"

//*-----------------------------------------------------------------------------
//* \fn    : AT91F_DF_wait_ready
//* \brief 
//*-----------------------------------------------------------------------------
char AT91F_DF_wait_ready(AT91PS_DF pDataFlash)
{
	unsigned int timeout = 0;

	while (timeout++ < AT91C_DF_TIMEOUT)
	{
		if (AT91F_DF_get_status(pDataFlash))
		{
			if (AT91F_DF_is_ready(pDataFlash))
				return 1;
		}
	}
	
	return 0;
}

//@<<AT91F_DF_send_command>>
//@+node:<<AT91F_DF_send_command>>
//*----------------------------------------------------------------------------
//* \fn    AT91F_DataFlashSendCommand
//* \brief Generic function to send a command to the dataflash
//*----------------------------------------------------------------------------
char AT91F_DF_send_command (
	AT91PS_DF pDataFlash,
	unsigned char bCmd,      // Command value
	unsigned char bCmdSize,  // Command Size
	char         *pData,     // Data to be sent
	unsigned int  dDataSize, // Data Size
	unsigned int  dAddress)  // Dataflash Address
{
 	unsigned int dInternalAdr;
 	AT91PS_PDC pPdc;

 	// Try to get the dataflash semaphore
	if ( (pDataFlash->bSemaphore) != UNLOCKED)
		return (char) 0;
	pDataFlash->bSemaphore = LOCKED;
    
	// Compute command pattern
	dInternalAdr = ((dAddress / AT91C_PAGE_SIZE(pDataFlash)) << AT91C_PAGE_OFFSET(pDataFlash)) \
 		+ (dAddress % AT91C_PAGE_SIZE(pDataFlash));
 		
 	if (AT91C_DF_NB_PAGE(pDataFlash) >= 16384)
	{
		pDataFlash->command[0] = (bCmd & 0x000000FF) | \
	                             ((dInternalAdr & 0x0F000000) >> 16) | \
	                             ((dInternalAdr & 0x00FF0000) >>  0) | \
	                             ((dInternalAdr & 0x0000FF00) << 16);
 		pDataFlash->command[1] =  (dInternalAdr & 0x000000FF);
	
		if ((bCmd != DB_CONTINUOUS_ARRAY_READ) && (bCmd != DB_PAGE_READ))
			bCmdSize++;
	}
	else
	{
		pDataFlash->command[0] = (bCmd & 0x000000FF) | \
	                             ((dInternalAdr & 0x00FF0000) >> 8) | \
	                             ((dInternalAdr & 0x0000FF00) << 8) | \
	                             ((dInternalAdr & 0x000000FF) << 24);
 		pDataFlash->command[1] = 0;
	}
	
 	// Send Command and data through the SPI
//	AT91F_SPI_CfgPCS(pDataFlash->pSpi, pDataFlash->bCs);
 	pPdc = (AT91PS_PDC) &(pDataFlash->pSpi->SPI_RPR);
 	AT91F_PDC_DisableRx(pPdc);
 	AT91F_PDC_SetRx(pPdc, (char *) &(pDataFlash->command), bCmdSize);
 	AT91F_PDC_SetNextRx(pPdc, pData, dDataSize);
    
	AT91F_PDC_DisableTx(pPdc);
	AT91F_PDC_SetTx(pPdc, (char *) &(pDataFlash->command), bCmdSize);
	AT91F_PDC_SetNextTx(pPdc, pData, dDataSize);

// 	AT91F_SPI_EnableIt(pDataFlash->pSpi, AT91C_SPI_RXBUFF);
 	AT91F_PDC_EnableRx(pPdc);
	AT91F_PDC_EnableTx(pPdc);

//$ NLe
    while (AT91F_DF_is_busy(pDataFlash) == LOCKED);
  
	return 1;
}
//@nonl
//@-node:<<AT91F_DF_send_command>>
//@nl

//@<<AT91F_DF_is_busy>>
//@+node:<<AT91F_DF_is_busy>>
//*----------------------------------------------------------------------------
//* \fn    AT91F_DataFlashHandler
//* \brief SPI Fixed Peripheral C interrupt handler.
//*----------------------------------------------------------------------------
AT91S_DF_SEM AT91F_DF_is_busy(
	AT91PS_DF pDataFlash)
{
	AT91PS_PDC pPdc;
	AT91PS_SPI pSpi = pDataFlash->pSpi;
	unsigned int dStatus = pSpi->SPI_SR;
	
//	dStatus = dStatus & pSpi->SPI_IMR;
	   
	//* If End of Receive Transfer interrupt occurred
 	if (( dStatus & AT91C_SPI_RXBUFF)) {            
		pPdc = (AT91PS_PDC) &(pDataFlash->pSpi->SPI_RPR);    
		// Disable the Transmit Interrupt
//		AT91F_SPI_DisableIt(pDataFlash->pSpi, AT91C_SPI_RXBUFF);
		AT91F_PDC_DisableTx(pPdc);
		AT91F_PDC_DisableRx(pPdc);
            
 		// Release the semaphore
		pDataFlash->bSemaphore = UNLOCKED;
		return UNLOCKED;
	}
	return  pDataFlash->bSemaphore;         		
}


//@-node:<<AT91F_DF_is_busy>>
//@nl
//@-node:@file dataflash.c
//@-leo
