//*----------------------------------------------------------------------------
//*         ATMEL Microcontroller Software Support  -  ROUSSET  -
//*----------------------------------------------------------------------------
//* The software is delivered "AS IS" without warranty or condition of any
//* kind, either express, implied or statutory. This includes without
//* limitation any warranty or condition with respect to merchantability or
//* fitness for any particular purpose, or against the infringements of
//* intellectual property rights of others.
//*----------------------------------------------------------------------------
//* File Name           : main.c
//*----------------------------------------------------------------------------

#include "main.h"
#include "dataflash.h"

// Define TCL_PAGE_BUF which is the address of the page content to write (written by the SAMBA TCL)

// TCL_PAGE_BUF[-1] = Link used (1 = DBGU     0 = else)
// TCL_PAGE_BUF[0] = DF Destination address
// TCL_PAGE_BUF[1] = DF Page Number (not necessary...)
// TCL_PAGE_BUF[2] = DF Page Size
// TCL_PAGE_BUF[3] = DF Page Offset
// TCL_PAGE_BUF[4] = pData

unsigned int *TCL_PAGE_BUF = (unsigned int *)PAGE_BUF;

#define R_BIT	1
#define W_BIT	2

//*----------------------------------------------------------------------------
//* Function Name       : read_jtag_control
//* Object              : read debug communication control register
//* Input Parameters    :
//* Output Parameters   : R0
//*----------------------------------------------------------------------------
__inline unsigned int read_jtag_control(void)
{
    /* read debug communication channel control register in R0*/
    __asm("MRC p14, 0, r0, c0, c0");
    /*return value = (R0)*/
}

//*----------------------------------------------------------------------------
//* Function Name       : write_jtag
//* Object              : write data on jtag
//*----------------------------------------------------------------------------
__inline void write_jtag_data(unsigned int val)
{
    /* write debug communication channel control register*/
    __asm("MCR p14, 0, val, c1, c0");
}

//*----------------------------------------------------------------------------
//* Function Name       : main
//* Object              :
//*----------------------------------------------------------------------------
int main()
{
	AT91S_DF sDF;
	AT91PS_DF pDf = (AT91PS_DF)&sDF;
#ifdef DBG_ACK
	unsigned int dCom = *(TCL_PAGE_BUF - 1);
#endif
	unsigned int dAddress = *(TCL_PAGE_BUF);
	char *pData = (char *)(TCL_PAGE_BUF+4);

	pDf->pSpi = AT91C_BASE_SPI0;
	pDf->bSemaphore = UNLOCKED;
	pDf->dfDescription.pages_number = 	*(TCL_PAGE_BUF+1);
	pDf->dfDescription.pages_size = 	*(TCL_PAGE_BUF+2);
	pDf->dfDescription.page_offset = 	*(TCL_PAGE_BUF+3);

#if defined (AT91SAM9260) || defined (AT91SAM9261) || defined (AT91SAM9263) || defined (AT91SAM9XE128) || defined (AT91SAM9XE256) || defined (AT91SAM9XE512) || defined (AT91SAM9RL64) || defined (AT91CAP9)
	// Enable PMC SPI_Clock
	AT91f_OpenSpiPIO();
	AT91C_BASE_PMC->PMC_PCER = 1 << AT91C_ID_SPI0;
#endif

#ifdef DF_WRITE

	AT91F_DF_write_buf1(pDf, pData, AT91C_PAGE_SIZE(pDf), dAddress);
	AT91F_DF_wait_ready(pDf);
	AT91F_DF_page_erase(pDf, dAddress);
	AT91F_DF_wait_ready(pDf);
	AT91F_DF_pgm_buf1(pDf, pData, AT91C_PAGE_SIZE(pDf), dAddress);
	AT91F_DF_wait_ready(pDf);

#endif

#ifdef DF_READ
	AT91F_DF_wait_ready(pDf);
	AT91F_DF_page_read(pDf, pData, AT91C_PAGE_SIZE(pDf), dAddress);
	AT91F_DF_wait_ready(pDf);
#endif

#ifdef DBG_ACK
	// Send ACK char for serial link
	if (dCom == 1)
		AT91C_BASE_DBGU->DBGU_THR = 0x6;
	// Send ACK char for JTAG link
	if (dCom == 2)
	{
		while((read_jtag_control() & W_BIT)); // Wait a new data to write
		write_jtag_data(0x6);
	}

#endif

	return 1;
}

