//*----------------------------------------------------------------------------
//*         ATMEL Microcontroller Software Support  -  ROUSSET  -
//*----------------------------------------------------------------------------
//* The software is delivered "AS IS" without warranty or condition of any
//* kind, either express, implied or statutory. This includes without
//* limitation any warranty or condition with respect to merchantability or
//* fitness for any particular purpose, or against the infringements of
//* intellectual property rights of others.
//*----------------------------------------------------------------------------
//* File Name           : main.c
//*----------------------------------------------------------------------------

#include "main.h"

// TCL_PAGE_BUF[-1] = Link used (1 = DBGU     0 = else)
// TCL_PAGE_BUF[0] = Buffer

#define R_BIT	1
#define W_BIT	2

//*----------------------------------------------------------------------------
//* Function Name       : read_jtag_control
//* Object              : read debug communication control register
//* Input Parameters    :
//* Output Parameters   : R0
//*----------------------------------------------------------------------------
__inline unsigned int read_jtag_control(void)
{
    /* read debug communication channel control register in R0*/
    __asm("MRC p14, 0, r0, c0, c0");
    /*return value = (R0)*/
}

//*----------------------------------------------------------------------------
//* Function Name       : write_jtag
//* Object              : write data on jtag
//*----------------------------------------------------------------------------
__inline void write_jtag_data(unsigned int val)
{
    /* write debug communication channel control register*/
    __asm("MCR p14, 0, val, c1, c0");
}

int main()
{
	unsigned int j;
	// Define TCL_PAGE_NB which is the address of the page number to write (written by the SAMBA TCL)
	unsigned int TCL_PAGE_NB = (unsigned int) *(TCL_PAGE_BUF + AT91C_PAGE_SIZE/4);
	unsigned int *FLASH_BASE_ADDR = (unsigned int *) (AT91C_IFLASH);
#ifdef DBG_ACK
	unsigned int dCom = *(TCL_PAGE_BUF - 1);
#endif

	unsigned int *load_address = FLASH_BASE_ADDR + (TCL_PAGE_NB * AT91C_PAGE_SIZE)/4;

	// Wait for End Of Programming
	while( !(AT91C_BASE_MC->MC_FSR & AT91C_MC_FRDY) );

	// Load the buffer
	for(j = 0;j < AT91C_PAGE_SIZE/4;j++){
		*(load_address + j) = *(TCL_PAGE_BUF + j);
	}

	// Send Page Program Command
	AT91C_BASE_MC->MC_FCR = (AT91C_MC_FCMD_START_PROG |  ((TCL_PAGE_NB << 8) & AT91C_MC_PAGEN) | (0x5A << 24));

#ifdef DBG_ACK
	// Send ACK char for serial link
	if (dCom == 1)
		AT91C_BASE_DBGU->DBGU_THR = 0x6;
	// Send ACK char for JTAG link
	if (dCom == 2)
	{
		while((read_jtag_control() & W_BIT)); // Wait a new data to write
		write_jtag_data(0x6);
	}
#endif

	return 1;
}

