//*----------------------------------------------------------------------------
//*         ATMEL Microcontroller Software Support  -  ROUSSET  -
//*----------------------------------------------------------------------------
//* The software is delivered "AS IS" without warranty or condition of any
//* kind, either express, implied or statutory. This includes without
//* limitation any warranty or condition with respect to merchantability or
//* fitness for any particular purpose, or against the infringements of
//* intellectual property rights of others.
//*----------------------------------------------------------------------------
//* File Name           : main_samEFC2.c
//* Creation            : 02/Mar/06 JPP
//*----------------------------------------------------------------------------

#ifdef MISTRAL_FL023
#define MISTRAL_FL023_BASE    ((AT91_REG *) 	0xFFFFF800) // (FL023) Config Register
#define MISTRAL_FL023_MASK_AND_WRITE                0x7FFFFFFF // Bit 31
#define MISTRAL_FL023_MASK_OR_WRITE                 0x80000000 // Bit 31
#endif


#include "main.h"

// TCL_PAGE_BUF[-1] = Link used (1 = DBGU     0 = else)
// TCL_PAGE_BUF[0] = Buffer

#define AT91C_FKEY   (0x5A << 24)


#define R_BIT	1
#define W_BIT	2

//*----------------------------------------------------------------------------
//* Function Name       : read_jtag_control
//* Object              : read debug communication control register
//* Input Parameters    :
//* Output Parameters   : R0
//*----------------------------------------------------------------------------
__inline unsigned int read_jtag_control(void)
{
    /* read debug communication channel control register in R0*/
    __asm("MRC p14, 0, r0, c0, c0");
    /*return value = (R0)*/
}

//*----------------------------------------------------------------------------
//* Function Name       : write_jtag
//* Object              : write data on jtag
//*----------------------------------------------------------------------------
__inline void write_jtag_data(unsigned int val)
{
    /* write debug communication channel control register*/
    __asm("MCR p14, 0, val, c1, c0");
}

int main()
{
	unsigned int j;

	// Define TCL_PAGE_NB which is the address of the page number to write (written by the SAMBA TCL)
	unsigned int TCL_PAGE_NB = (unsigned int) *(TCL_PAGE_BUF + AT91C_PAGE_SIZE/4);
	unsigned int *FLASH_BASE_ADDR = (unsigned int *) (AT91C_IFLASH);
#ifdef DBG_ACK
	unsigned int dCom = *(TCL_PAGE_BUF - 1);
#endif

	unsigned int *load_address = FLASH_BASE_ADDR + (TCL_PAGE_NB * AT91C_PAGE_SIZE)/4;

	// Wait for End Of Programming
	while( !(AT91C_BASE_EFC->EFC_FSR & AT91C_EFC_FRDY) );

	// Load the buffer
	for(j = 0;j < AT91C_PAGE_SIZE/4;j++){
		*(load_address + j) = *(TCL_PAGE_BUF + j);
	}

#ifdef MISTRAL_FL023
*MISTRAL_FL023_BASE &= MISTRAL_FL023_MASK_AND_WRITE;
#endif
	// Send Page Program Command
	AT91C_BASE_EFC->EFC_FCR = (AT91C_EFC_FCMD_WP |  ((TCL_PAGE_NB << 8) | AT91C_FKEY) );

#ifdef MISTRAL_FL023
*MISTRAL_FL023_BASE |= MISTRAL_FL023_MASK_OR_WRITE;
#endif

#ifdef DBG_ACK
	// Send ACK char for serial link
	if (dCom == 1)
		AT91C_BASE_DBGU->DBGU_THR = 0x6;
	// Send ACK char for JTAG link
	if (dCom == 2)
	{
		while((read_jtag_control() & W_BIT)); // Wait a new data to write
		write_jtag_data(0x6);
	}
#endif

	return 1;
}

